# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class ServerGroup(pulumi.CustomResource):
    """
    Manages a V2 Server Group resource within OpenStack.
    """
    def __init__(__self__, __name__, __opts__=None, name=None, policies=None, region=None, value_specs=None):
        """Create a ServerGroup resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if name and not isinstance(name, basestring):
            raise TypeError('Expected property name to be a basestring')
        __self__.name = name
        """
        A unique name for the server group. Changing this creates
        a new server group.
        """
        __props__['name'] = name

        if policies and not isinstance(policies, list):
            raise TypeError('Expected property policies to be a list')
        __self__.policies = policies
        """
        The set of policies for the server group. Only two
        two policies are available right now, and both are mutually exclusive. See
        the Policies section for more information. Changing this creates a new
        server group.
        """
        __props__['policies'] = policies

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to obtain the V2 Compute client.
        If omitted, the `region` argument of the provider is used. Changing
        this creates a new server group.
        """
        __props__['region'] = region

        if value_specs and not isinstance(value_specs, dict):
            raise TypeError('Expected property value_specs to be a dict')
        __self__.value_specs = value_specs
        """
        Map of additional options.
        """
        __props__['valueSpecs'] = value_specs

        __self__.members = pulumi.runtime.UNKNOWN
        """
        The instances that are part of this server group.
        """

        super(ServerGroup, __self__).__init__(
            'openstack:compute/serverGroup:ServerGroup',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'members' in outs:
            self.members = outs['members']
        if 'name' in outs:
            self.name = outs['name']
        if 'policies' in outs:
            self.policies = outs['policies']
        if 'region' in outs:
            self.region = outs['region']
        if 'valueSpecs' in outs:
            self.value_specs = outs['valueSpecs']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class GetFlavorResult(object):
    """
    A collection of values returned by getFlavor.
    """
    def __init__(__self__, is_public=None, region=None):
        if is_public and not isinstance(is_public, bool):
            raise TypeError('Expected argument is_public to be a bool')
        __self__.is_public = is_public
        """
        Whether the flavor is public or private.
        """
        if region and not isinstance(region, basestring):
            raise TypeError('Expected argument region to be a basestring')
        __self__.region = region

def get_flavor(disk=None, min_disk=None, min_ram=None, name=None, ram=None, region=None, rx_tx_factor=None, swap=None, vcpus=None):
    """
    Use this data source to get the ID of an available OpenStack flavor.
    """
    __args__ = dict()

    __args__['disk'] = disk
    __args__['minDisk'] = min_disk
    __args__['minRam'] = min_ram
    __args__['name'] = name
    __args__['ram'] = ram
    __args__['region'] = region
    __args__['rxTxFactor'] = rx_tx_factor
    __args__['swap'] = swap
    __args__['vcpus'] = vcpus
    __ret__ = pulumi.runtime.invoke('openstack:compute/getFlavor:getFlavor', __args__)

    return GetFlavorResult(
        is_public=__ret__.get('isPublic'),
        region=__ret__.get('region'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class RoleAssignment(pulumi.CustomResource):
    """
    Manages a V3 Role assignment within OpenStack Keystone.
    
    Note: You _must_ have admin privileges in your OpenStack cloud to use
    this resource.
    """
    def __init__(__self__, __name__, __opts__=None, domain_id=None, group_id=None, project_id=None, role_id=None, user_id=None):
        """Create a RoleAssignment resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if domain_id and not isinstance(domain_id, basestring):
            raise TypeError('Expected property domain_id to be a basestring')
        __self__.domain_id = domain_id
        """
        The domain to assign the role in.
        """
        __props__['domainId'] = domain_id

        if group_id and not isinstance(group_id, basestring):
            raise TypeError('Expected property group_id to be a basestring')
        __self__.group_id = group_id
        """
        The group to assign the role to.
        """
        __props__['groupId'] = group_id

        if project_id and not isinstance(project_id, basestring):
            raise TypeError('Expected property project_id to be a basestring')
        __self__.project_id = project_id
        """
        The project to assign the role in.
        """
        __props__['projectId'] = project_id

        if not role_id:
            raise TypeError('Missing required property role_id')
        elif not isinstance(role_id, basestring):
            raise TypeError('Expected property role_id to be a basestring')
        __self__.role_id = role_id
        """
        The role to assign.
        """
        __props__['roleId'] = role_id

        if user_id and not isinstance(user_id, basestring):
            raise TypeError('Expected property user_id to be a basestring')
        __self__.user_id = user_id
        """
        The user to assign the role to.
        """
        __props__['userId'] = user_id

        super(RoleAssignment, __self__).__init__(
            'openstack:identity/roleAssignment:RoleAssignment',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'domainId' in outs:
            self.domain_id = outs['domainId']
        if 'groupId' in outs:
            self.group_id = outs['groupId']
        if 'projectId' in outs:
            self.project_id = outs['projectId']
        if 'roleId' in outs:
            self.role_id = outs['roleId']
        if 'userId' in outs:
            self.user_id = outs['userId']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class SecGroup(pulumi.CustomResource):
    """
    Manages a V2 neutron security group resource within OpenStack.
    Unlike Nova security groups, neutron separates the group from the rules
    and also allows an admin to target a specific tenant_id.
    """
    def __init__(__self__, __name__, __opts__=None, delete_default_rules=None, description=None, name=None, region=None, tenant_id=None):
        """Create a SecGroup resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if delete_default_rules and not isinstance(delete_default_rules, bool):
            raise TypeError('Expected property delete_default_rules to be a bool')
        __self__.delete_default_rules = delete_default_rules
        """
        Whether or not to delete the default
        egress security rules. This is `false` by default. See the below note
        for more information.
        """
        __props__['deleteDefaultRules'] = delete_default_rules

        if description and not isinstance(description, basestring):
            raise TypeError('Expected property description to be a basestring')
        __self__.description = description
        """
        A unique name for the security group.
        """
        __props__['description'] = description

        if name and not isinstance(name, basestring):
            raise TypeError('Expected property name to be a basestring')
        __self__.name = name
        """
        A unique name for the security group.
        """
        __props__['name'] = name

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to obtain the V2 networking client.
        A networking client is needed to create a port. If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        security group.
        """
        __props__['region'] = region

        if tenant_id and not isinstance(tenant_id, basestring):
            raise TypeError('Expected property tenant_id to be a basestring')
        __self__.tenant_id = tenant_id
        """
        The owner of the security group. Required if admin
        wants to create a port for another tenant. Changing this creates a new
        security group.
        """
        __props__['tenantId'] = tenant_id

        super(SecGroup, __self__).__init__(
            'openstack:networking/secGroup:SecGroup',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'deleteDefaultRules' in outs:
            self.delete_default_rules = outs['deleteDefaultRules']
        if 'description' in outs:
            self.description = outs['description']
        if 'name' in outs:
            self.name = outs['name']
        if 'region' in outs:
            self.region = outs['region']
        if 'tenantId' in outs:
            self.tenant_id = outs['tenantId']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class Configuration(pulumi.CustomResource):
    """
    Manages a V1 DB configuration resource within OpenStack.
    """
    def __init__(__self__, __name__, __opts__=None, configurations=None, datastore=None, description=None, name=None, region=None):
        """Create a Configuration resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if configurations and not isinstance(configurations, list):
            raise TypeError('Expected property configurations to be a list')
        __self__.configurations = configurations
        """
        An array of configuration parameter name and value. Can be specified multiple times. The configuration object structure is documented below.
        """
        __props__['configurations'] = configurations

        if not datastore:
            raise TypeError('Missing required property datastore')
        elif not isinstance(datastore, dict):
            raise TypeError('Expected property datastore to be a dict')
        __self__.datastore = datastore
        """
        An array of database engine type and version. The datastore
        object structure is documented below. Changing this creates resource.
        """
        __props__['datastore'] = datastore

        if not description:
            raise TypeError('Missing required property description')
        elif not isinstance(description, basestring):
            raise TypeError('Expected property description to be a basestring')
        __self__.description = description
        """
        Description of the resource.
        """
        __props__['description'] = description

        if name and not isinstance(name, basestring):
            raise TypeError('Expected property name to be a basestring')
        __self__.name = name
        """
        Configuration parameter name. Changing this creates a new resource.
        """
        __props__['name'] = name

        if not region:
            raise TypeError('Missing required property region')
        elif not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to create the db instance. Changing this
        creates a new instance.
        """
        __props__['region'] = region

        super(Configuration, __self__).__init__(
            'openstack:database/configuration:Configuration',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'configurations' in outs:
            self.configurations = outs['configurations']
        if 'datastore' in outs:
            self.datastore = outs['datastore']
        if 'description' in outs:
            self.description = outs['description']
        if 'name' in outs:
            self.name = outs['name']
        if 'region' in outs:
            self.region = outs['region']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class SubnetPool(pulumi.CustomResource):
    """
    Manages a V2 Neutron subnetpool resource within OpenStack.
    """
    def __init__(__self__, __name__, __opts__=None, address_scope_id=None, default_prefixlen=None, default_quota=None, description=None, ip_version=None, is_default=None, max_prefixlen=None, min_prefixlen=None, name=None, prefixes=None, project_id=None, region=None, shared=None, value_specs=None):
        """Create a SubnetPool resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if address_scope_id and not isinstance(address_scope_id, basestring):
            raise TypeError('Expected property address_scope_id to be a basestring')
        __self__.address_scope_id = address_scope_id
        """
        The Neutron address scope to assign to the
        subnetpool. Changing this updates the address scope id of the existing
        subnetpool.
        """
        __props__['addressScopeId'] = address_scope_id

        if default_prefixlen and not isinstance(default_prefixlen, int):
            raise TypeError('Expected property default_prefixlen to be a int')
        __self__.default_prefixlen = default_prefixlen
        """
        The size of the prefix to allocate when the cidr
        or prefixlen attributes are omitted when you create the subnet. Defaults to the
        MinPrefixLen. Changing this updates the default prefixlen of the existing
        subnetpool.
        """
        __props__['defaultPrefixlen'] = default_prefixlen

        if default_quota and not isinstance(default_quota, int):
            raise TypeError('Expected property default_quota to be a int')
        __self__.default_quota = default_quota
        """
        The per-project quota on the prefix space that can be
        allocated from the subnetpool for project subnets. Changing this updates the
        default quota of the existing subnetpool.
        """
        __props__['defaultQuota'] = default_quota

        if description and not isinstance(description, basestring):
            raise TypeError('Expected property description to be a basestring')
        __self__.description = description
        """
        The human-readable description for the subnetpool.
        Changing this updates the description of the existing subnetpool.
        """
        __props__['description'] = description

        if ip_version and not isinstance(ip_version, int):
            raise TypeError('Expected property ip_version to be a int')
        __self__.ip_version = ip_version
        """
        The IP protocol version.
        """
        __props__['ipVersion'] = ip_version

        if is_default and not isinstance(is_default, bool):
            raise TypeError('Expected property is_default to be a bool')
        __self__.is_default = is_default
        """
        Indicates whether the subnetpool is default
        subnetpool or not. Changing this updates the default status of the existing
        subnetpool.
        """
        __props__['isDefault'] = is_default

        if max_prefixlen and not isinstance(max_prefixlen, int):
            raise TypeError('Expected property max_prefixlen to be a int')
        __self__.max_prefixlen = max_prefixlen
        """
        The maximum prefix size that can be allocated from
        the subnetpool. For IPv4 subnetpools, default is 32. For IPv6 subnetpools,
        default is 128. Changing this updates the max prefixlen of the existing
        subnetpool.
        """
        __props__['maxPrefixlen'] = max_prefixlen

        if min_prefixlen and not isinstance(min_prefixlen, int):
            raise TypeError('Expected property min_prefixlen to be a int')
        __self__.min_prefixlen = min_prefixlen
        """
        The smallest prefix that can be allocated from a
        subnetpool. For IPv4 subnetpools, default is 8. For IPv6 subnetpools, default
        is 64. Changing this updates the min prefixlen of the existing subnetpool.
        """
        __props__['minPrefixlen'] = min_prefixlen

        if name and not isinstance(name, basestring):
            raise TypeError('Expected property name to be a basestring')
        __self__.name = name
        """
        The name of the subnetpool. Changing this updates the name of
        the existing subnetpool.
        """
        __props__['name'] = name

        if not prefixes:
            raise TypeError('Missing required property prefixes')
        elif not isinstance(prefixes, list):
            raise TypeError('Expected property prefixes to be a list')
        __self__.prefixes = prefixes
        """
        A list of subnet prefixes to assign to the subnetpool.
        Neutron API merges adjacent prefixes and treats them as a single prefix. Each
        subnet prefix must be unique among all subnet prefixes in all subnetpools that
        are associated with the address scope. Changing this updates the prefixes list
        of the existing subnetpool.
        """
        __props__['prefixes'] = prefixes

        if project_id and not isinstance(project_id, basestring):
            raise TypeError('Expected property project_id to be a basestring')
        __self__.project_id = project_id
        """
        The owner of the subnetpool. Required if admin wants to
        create a subnetpool for another project. Changing this creates a new subnetpool.
        """
        __props__['projectId'] = project_id

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to obtain the V2 Networking client.
        A Networking client is needed to create a Neutron subnetpool. If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        subnetpool.
        """
        __props__['region'] = region

        if shared and not isinstance(shared, bool):
            raise TypeError('Expected property shared to be a bool')
        __self__.shared = shared
        """
        Indicates whether this subnetpool is shared across
        all projects. Changing this updates the shared status of the existing
        subnetpool.
        """
        __props__['shared'] = shared

        if value_specs and not isinstance(value_specs, dict):
            raise TypeError('Expected property value_specs to be a dict')
        __self__.value_specs = value_specs
        """
        Map of additional options.
        """
        __props__['valueSpecs'] = value_specs

        __self__.created_at = pulumi.runtime.UNKNOWN
        """
        The time at which subnetpool was created.
        """
        __self__.revision_number = pulumi.runtime.UNKNOWN
        """
        The revision number of the subnetpool.
        """
        __self__.updated_at = pulumi.runtime.UNKNOWN
        """
        The time at which subnetpool was created.
        """

        super(SubnetPool, __self__).__init__(
            'openstack:networking/subnetPool:SubnetPool',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'addressScopeId' in outs:
            self.address_scope_id = outs['addressScopeId']
        if 'createdAt' in outs:
            self.created_at = outs['createdAt']
        if 'defaultPrefixlen' in outs:
            self.default_prefixlen = outs['defaultPrefixlen']
        if 'defaultQuota' in outs:
            self.default_quota = outs['defaultQuota']
        if 'description' in outs:
            self.description = outs['description']
        if 'ipVersion' in outs:
            self.ip_version = outs['ipVersion']
        if 'isDefault' in outs:
            self.is_default = outs['isDefault']
        if 'maxPrefixlen' in outs:
            self.max_prefixlen = outs['maxPrefixlen']
        if 'minPrefixlen' in outs:
            self.min_prefixlen = outs['minPrefixlen']
        if 'name' in outs:
            self.name = outs['name']
        if 'prefixes' in outs:
            self.prefixes = outs['prefixes']
        if 'projectId' in outs:
            self.project_id = outs['projectId']
        if 'region' in outs:
            self.region = outs['region']
        if 'revisionNumber' in outs:
            self.revision_number = outs['revisionNumber']
        if 'shared' in outs:
            self.shared = outs['shared']
        if 'updatedAt' in outs:
            self.updated_at = outs['updatedAt']
        if 'valueSpecs' in outs:
            self.value_specs = outs['valueSpecs']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class GetSecGroupResult(object):
    """
    A collection of values returned by getSecGroup.
    """
    def __init__(__self__, region=None, tenant_id=None):
        if region and not isinstance(region, basestring):
            raise TypeError('Expected argument region to be a basestring')
        __self__.region = region
        """
        See Argument Reference above.
        """
        if tenant_id and not isinstance(tenant_id, basestring):
            raise TypeError('Expected argument tenant_id to be a basestring')
        __self__.tenant_id = tenant_id

def get_sec_group(name=None, region=None, secgroup_id=None, tenant_id=None):
    """
    Use this data source to get the ID of an available OpenStack security group.
    """
    __args__ = dict()

    __args__['name'] = name
    __args__['region'] = region
    __args__['secgroupId'] = secgroup_id
    __args__['tenantId'] = tenant_id
    __ret__ = pulumi.runtime.invoke('openstack:networking/getSecGroup:getSecGroup', __args__)

    return GetSecGroupResult(
        region=__ret__.get('region'),
        tenant_id=__ret__.get('tenantId'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class VolumeAttach(pulumi.CustomResource):
    """
    Attaches a Block Storage Volume to an Instance using the OpenStack
    Compute (Nova) v2 API.
    """
    def __init__(__self__, __name__, __opts__=None, device=None, instance_id=None, region=None, volume_id=None):
        """Create a VolumeAttach resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if device and not isinstance(device, basestring):
            raise TypeError('Expected property device to be a basestring')
        __self__.device = device
        """
        The device of the volume attachment (ex: `/dev/vdc`).
        _NOTE_: Being able to specify a device is dependent upon the hypervisor in
        use. There is a chance that the device specified in Terraform will not be
        the same device the hypervisor chose. If this happens, Terraform will wish
        to update the device upon subsequent applying which will cause the volume
        to be detached and reattached indefinitely. Please use with caution.
        """
        __props__['device'] = device

        if not instance_id:
            raise TypeError('Missing required property instance_id')
        elif not isinstance(instance_id, basestring):
            raise TypeError('Expected property instance_id to be a basestring')
        __self__.instance_id = instance_id
        """
        The ID of the Instance to attach the Volume to.
        """
        __props__['instanceId'] = instance_id

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to obtain the V2 Compute client.
        A Compute client is needed to create a volume attachment. If omitted, the
        `region` argument of the provider is used. Changing this creates a
        new volume attachment.
        """
        __props__['region'] = region

        if not volume_id:
            raise TypeError('Missing required property volume_id')
        elif not isinstance(volume_id, basestring):
            raise TypeError('Expected property volume_id to be a basestring')
        __self__.volume_id = volume_id
        """
        The ID of the Volume to attach to an Instance.
        """
        __props__['volumeId'] = volume_id

        super(VolumeAttach, __self__).__init__(
            'openstack:compute/volumeAttach:VolumeAttach',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'device' in outs:
            self.device = outs['device']
        if 'instanceId' in outs:
            self.instance_id = outs['instanceId']
        if 'region' in outs:
            self.region = outs['region']
        if 'volumeId' in outs:
            self.volume_id = outs['volumeId']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class Pool(pulumi.CustomResource):
    """
    Manages a V2 pool resource within OpenStack.
    """
    def __init__(__self__, __name__, __opts__=None, admin_state_up=None, description=None, lb_method=None, listener_id=None, loadbalancer_id=None, name=None, persistences=None, protocol=None, region=None, tenant_id=None):
        """Create a Pool resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if admin_state_up and not isinstance(admin_state_up, bool):
            raise TypeError('Expected property admin_state_up to be a bool')
        __self__.admin_state_up = admin_state_up
        """
        The administrative state of the pool.
        A valid value is true (UP) or false (DOWN).
        """
        __props__['adminStateUp'] = admin_state_up

        if description and not isinstance(description, basestring):
            raise TypeError('Expected property description to be a basestring')
        __self__.description = description
        """
        Human-readable description for the pool.
        """
        __props__['description'] = description

        if not lb_method:
            raise TypeError('Missing required property lb_method')
        elif not isinstance(lb_method, basestring):
            raise TypeError('Expected property lb_method to be a basestring')
        __self__.lb_method = lb_method
        """
        The load balancing algorithm to
        distribute traffic to the pool's members. Must be one of
        ROUND_ROBIN, LEAST_CONNECTIONS, or SOURCE_IP.
        """
        __props__['lbMethod'] = lb_method

        if listener_id and not isinstance(listener_id, basestring):
            raise TypeError('Expected property listener_id to be a basestring')
        __self__.listener_id = listener_id
        """
        The Listener on which the members of the pool
        will be associated with. Changing this creates a new pool.
        Note:  One of LoadbalancerID or ListenerID must be provided.
        """
        __props__['listenerId'] = listener_id

        if loadbalancer_id and not isinstance(loadbalancer_id, basestring):
            raise TypeError('Expected property loadbalancer_id to be a basestring')
        __self__.loadbalancer_id = loadbalancer_id
        """
        The load balancer on which to provision this
        pool. Changing this creates a new pool.
        Note:  One of LoadbalancerID or ListenerID must be provided.
        """
        __props__['loadbalancerId'] = loadbalancer_id

        if name and not isinstance(name, basestring):
            raise TypeError('Expected property name to be a basestring')
        __self__.name = name
        """
        Human-readable name for the pool.
        """
        __props__['name'] = name

        if persistences and not isinstance(persistences, list):
            raise TypeError('Expected property persistences to be a list')
        __self__.persistences = persistences
        """
        Omit this field to prevent session persistence.  Indicates
        whether connections in the same session will be processed by the same Pool
        member or not. Changing this creates a new pool.
        """
        __props__['persistences'] = persistences

        if not protocol:
            raise TypeError('Missing required property protocol')
        elif not isinstance(protocol, basestring):
            raise TypeError('Expected property protocol to be a basestring')
        __self__.protocol = protocol
        """
        See Argument Reference above.
        """
        __props__['protocol'] = protocol

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to obtain the V2 Networking client.
        A Networking client is needed to create an . If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        pool.
        """
        __props__['region'] = region

        if tenant_id and not isinstance(tenant_id, basestring):
            raise TypeError('Expected property tenant_id to be a basestring')
        __self__.tenant_id = tenant_id
        """
        Required for admins. The UUID of the tenant who owns
        the pool.  Only administrative users can specify a tenant UUID
        other than their own. Changing this creates a new pool.
        """
        __props__['tenantId'] = tenant_id

        super(Pool, __self__).__init__(
            'openstack:loadbalancer/pool:Pool',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'adminStateUp' in outs:
            self.admin_state_up = outs['adminStateUp']
        if 'description' in outs:
            self.description = outs['description']
        if 'lbMethod' in outs:
            self.lb_method = outs['lbMethod']
        if 'listenerId' in outs:
            self.listener_id = outs['listenerId']
        if 'loadbalancerId' in outs:
            self.loadbalancer_id = outs['loadbalancerId']
        if 'name' in outs:
            self.name = outs['name']
        if 'persistences' in outs:
            self.persistences = outs['persistences']
        if 'protocol' in outs:
            self.protocol = outs['protocol']
        if 'region' in outs:
            self.region = outs['region']
        if 'tenantId' in outs:
            self.tenant_id = outs['tenantId']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class SecGroupRule(pulumi.CustomResource):
    """
    Manages a V2 neutron security group rule resource within OpenStack.
    Unlike Nova security groups, neutron separates the group from the rules
    and also allows an admin to target a specific tenant_id.
    """
    def __init__(__self__, __name__, __opts__=None, direction=None, ethertype=None, port_range_max=None, port_range_min=None, protocol=None, region=None, remote_group_id=None, remote_ip_prefix=None, security_group_id=None, tenant_id=None):
        """Create a SecGroupRule resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if not direction:
            raise TypeError('Missing required property direction')
        elif not isinstance(direction, basestring):
            raise TypeError('Expected property direction to be a basestring')
        __self__.direction = direction
        """
        The direction of the rule, valid values are __ingress__
        or __egress__. Changing this creates a new security group rule.
        """
        __props__['direction'] = direction

        if not ethertype:
            raise TypeError('Missing required property ethertype')
        elif not isinstance(ethertype, basestring):
            raise TypeError('Expected property ethertype to be a basestring')
        __self__.ethertype = ethertype
        """
        The layer 3 protocol type, valid values are __IPv4__
        or __IPv6__. Changing this creates a new security group rule.
        """
        __props__['ethertype'] = ethertype

        if port_range_max and not isinstance(port_range_max, int):
            raise TypeError('Expected property port_range_max to be a int')
        __self__.port_range_max = port_range_max
        """
        The higher part of the allowed port range, valid
        integer value needs to be between 1 and 65535. Changing this creates a new
        security group rule.
        """
        __props__['portRangeMax'] = port_range_max

        if port_range_min and not isinstance(port_range_min, int):
            raise TypeError('Expected property port_range_min to be a int')
        __self__.port_range_min = port_range_min
        """
        The lower part of the allowed port range, valid
        integer value needs to be between 1 and 65535. Changing this creates a new
        security group rule.
        """
        __props__['portRangeMin'] = port_range_min

        if protocol and not isinstance(protocol, basestring):
            raise TypeError('Expected property protocol to be a basestring')
        __self__.protocol = protocol
        """
        The layer 4 protocol type, valid values are following. Changing this creates a new security group rule. This is required if you want to specify a port range.
        * __tcp__
        * __udp__
        * __icmp__
        * __ah__
        * __dccp__
        * __egp__
        * __esp__
        * __gre__
        * __igmp__
        * __ipv6-encap__
        * __ipv6-frag__
        * __ipv6-icmp__
        * __ipv6-nonxt__
        * __ipv6-opts__
        * __ipv6-route__
        * __ospf__
        * __pgm__
        * __rsvp__
        * __sctp__
        * __udplite__
        * __vrrp__
        """
        __props__['protocol'] = protocol

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to obtain the V2 networking client.
        A networking client is needed to create a port. If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        security group rule.
        """
        __props__['region'] = region

        if remote_group_id and not isinstance(remote_group_id, basestring):
            raise TypeError('Expected property remote_group_id to be a basestring')
        __self__.remote_group_id = remote_group_id
        """
        The remote group id, the value needs to be an
        Openstack ID of a security group in the same tenant. Changing this creates
        a new security group rule.
        """
        __props__['remoteGroupId'] = remote_group_id

        if remote_ip_prefix and not isinstance(remote_ip_prefix, basestring):
            raise TypeError('Expected property remote_ip_prefix to be a basestring')
        __self__.remote_ip_prefix = remote_ip_prefix
        """
        The remote CIDR, the value needs to be a valid
        CIDR (i.e. 192.168.0.0/16). Changing this creates a new security group rule.
        """
        __props__['remoteIpPrefix'] = remote_ip_prefix

        if not security_group_id:
            raise TypeError('Missing required property security_group_id')
        elif not isinstance(security_group_id, basestring):
            raise TypeError('Expected property security_group_id to be a basestring')
        __self__.security_group_id = security_group_id
        """
        The security group id the rule should belong
        to, the value needs to be an Openstack ID of a security group in the same
        tenant. Changing this creates a new security group rule.
        """
        __props__['securityGroupId'] = security_group_id

        if tenant_id and not isinstance(tenant_id, basestring):
            raise TypeError('Expected property tenant_id to be a basestring')
        __self__.tenant_id = tenant_id
        """
        The owner of the security group. Required if admin
        wants to create a port for another tenant. Changing this creates a new
        security group rule.
        """
        __props__['tenantId'] = tenant_id

        super(SecGroupRule, __self__).__init__(
            'openstack:networking/secGroupRule:SecGroupRule',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'direction' in outs:
            self.direction = outs['direction']
        if 'ethertype' in outs:
            self.ethertype = outs['ethertype']
        if 'portRangeMax' in outs:
            self.port_range_max = outs['portRangeMax']
        if 'portRangeMin' in outs:
            self.port_range_min = outs['portRangeMin']
        if 'protocol' in outs:
            self.protocol = outs['protocol']
        if 'region' in outs:
            self.region = outs['region']
        if 'remoteGroupId' in outs:
            self.remote_group_id = outs['remoteGroupId']
        if 'remoteIpPrefix' in outs:
            self.remote_ip_prefix = outs['remoteIpPrefix']
        if 'securityGroupId' in outs:
            self.security_group_id = outs['securityGroupId']
        if 'tenantId' in outs:
            self.tenant_id = outs['tenantId']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class FloatingIp(pulumi.CustomResource):
    """
    Manages a V2 floating IP resource within OpenStack Neutron (networking)
    that can be used for load balancers.
    These are similar to Nova (compute) floating IP resources,
    but only compute floating IPs can be used with compute instances.
    """
    def __init__(__self__, __name__, __opts__=None, fixed_ip=None, pool=None, port_id=None, region=None, subnet_id=None, tenant_id=None, value_specs=None):
        """Create a FloatingIp resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if fixed_ip and not isinstance(fixed_ip, basestring):
            raise TypeError('Expected property fixed_ip to be a basestring')
        __self__.fixed_ip = fixed_ip
        """
        Fixed IP of the port to associate with this floating IP. Required if
        the port has multiple fixed IPs.
        """
        __props__['fixedIp'] = fixed_ip

        if not pool:
            raise TypeError('Missing required property pool')
        elif not isinstance(pool, basestring):
            raise TypeError('Expected property pool to be a basestring')
        __self__.pool = pool
        """
        The name of the pool from which to obtain the floating
        IP. Changing this creates a new floating IP.
        """
        __props__['pool'] = pool

        if port_id and not isinstance(port_id, basestring):
            raise TypeError('Expected property port_id to be a basestring')
        __self__.port_id = port_id
        """
        ID of an existing port with at least one IP address to
        associate with this floating IP.
        """
        __props__['portId'] = port_id

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to obtain the V2 Networking client.
        A Networking client is needed to create a floating IP that can be used with
        another networking resource, such as a load balancer. If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        floating IP (which may or may not have a different address).
        """
        __props__['region'] = region

        if subnet_id and not isinstance(subnet_id, basestring):
            raise TypeError('Expected property subnet_id to be a basestring')
        __self__.subnet_id = subnet_id
        """
        The subnet ID of the floating IP pool. Specify this if
        the floating IP network has multiple subnets.
        """
        __props__['subnetId'] = subnet_id

        if tenant_id and not isinstance(tenant_id, basestring):
            raise TypeError('Expected property tenant_id to be a basestring')
        __self__.tenant_id = tenant_id
        """
        The target tenant ID in which to allocate the floating
        IP, if you specify this together with a port_id, make sure the target port
        belongs to the same tenant. Changing this creates a new floating IP (which
        may or may not have a different address)
        """
        __props__['tenantId'] = tenant_id

        if value_specs and not isinstance(value_specs, dict):
            raise TypeError('Expected property value_specs to be a dict')
        __self__.value_specs = value_specs
        """
        Map of additional options.
        """
        __props__['valueSpecs'] = value_specs

        __self__.address = pulumi.runtime.UNKNOWN
        """
        The actual floating IP address itself.
        """

        super(FloatingIp, __self__).__init__(
            'openstack:networking/floatingIp:FloatingIp',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'address' in outs:
            self.address = outs['address']
        if 'fixedIp' in outs:
            self.fixed_ip = outs['fixedIp']
        if 'pool' in outs:
            self.pool = outs['pool']
        if 'portId' in outs:
            self.port_id = outs['portId']
        if 'region' in outs:
            self.region = outs['region']
        if 'subnetId' in outs:
            self.subnet_id = outs['subnetId']
        if 'tenantId' in outs:
            self.tenant_id = outs['tenantId']
        if 'valueSpecs' in outs:
            self.value_specs = outs['valueSpecs']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

# Export this package's modules as members:
from project import *
from role_assignment import *
from role import *
from user import *
from get_auth_scope import *
from get_project import *
from get_role import *
from get_user import *



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class Project(pulumi.CustomResource):
    """
    Manages a V3 Project resource within OpenStack Keystone.
    
    Note: You _must_ have admin privileges in your OpenStack cloud to use
    this resource.
    """
    def __init__(__self__, __name__, __opts__=None, description=None, domain_id=None, enabled=None, is_domain=None, name=None, parent_id=None, region=None):
        """Create a Project resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if description and not isinstance(description, basestring):
            raise TypeError('Expected property description to be a basestring')
        __self__.description = description
        """
        A description of the project.
        """
        __props__['description'] = description

        if domain_id and not isinstance(domain_id, basestring):
            raise TypeError('Expected property domain_id to be a basestring')
        __self__.domain_id = domain_id
        """
        The domain this project belongs to.
        """
        __props__['domainId'] = domain_id

        if enabled and not isinstance(enabled, bool):
            raise TypeError('Expected property enabled to be a bool')
        __self__.enabled = enabled
        """
        Whether the project is enabled or disabled. Valid
        values are `true` and `false`.
        """
        __props__['enabled'] = enabled

        if is_domain and not isinstance(is_domain, bool):
            raise TypeError('Expected property is_domain to be a bool')
        __self__.is_domain = is_domain
        """
        Whether this project is a domain. Valid values
        are `true` and `false`.
        """
        __props__['isDomain'] = is_domain

        if name and not isinstance(name, basestring):
            raise TypeError('Expected property name to be a basestring')
        __self__.name = name
        """
        The name of the project.
        """
        __props__['name'] = name

        if parent_id and not isinstance(parent_id, basestring):
            raise TypeError('Expected property parent_id to be a basestring')
        __self__.parent_id = parent_id
        """
        The parent of this project.
        """
        __props__['parentId'] = parent_id

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to obtain the V3 Keystone client.
        If omitted, the `region` argument of the provider is used. Changing this
        creates a new User.
        """
        __props__['region'] = region

        super(Project, __self__).__init__(
            'openstack:identity/project:Project',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'description' in outs:
            self.description = outs['description']
        if 'domainId' in outs:
            self.domain_id = outs['domainId']
        if 'enabled' in outs:
            self.enabled = outs['enabled']
        if 'isDomain' in outs:
            self.is_domain = outs['isDomain']
        if 'name' in outs:
            self.name = outs['name']
        if 'parentId' in outs:
            self.parent_id = outs['parentId']
        if 'region' in outs:
            self.region = outs['region']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class User(pulumi.CustomResource):
    """
    Manages a V1 DB user resource within OpenStack.
    """
    def __init__(__self__, __name__, __opts__=None, databases=None, host=None, instance_id=None, name=None, password=None, region=None):
        """Create a User resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if databases and not isinstance(databases, list):
            raise TypeError('Expected property databases to be a list')
        __self__.databases = databases
        """
        A list of database user should have access to.
        """
        __props__['databases'] = databases

        if host and not isinstance(host, basestring):
            raise TypeError('Expected property host to be a basestring')
        __self__.host = host
        __props__['host'] = host

        if not instance_id:
            raise TypeError('Missing required property instance_id')
        elif not isinstance(instance_id, basestring):
            raise TypeError('Expected property instance_id to be a basestring')
        __self__.instance_id = instance_id
        __props__['instanceId'] = instance_id

        if name and not isinstance(name, basestring):
            raise TypeError('Expected property name to be a basestring')
        __self__.name = name
        """
        A unique name for the resource.
        """
        __props__['name'] = name

        if not password:
            raise TypeError('Missing required property password')
        elif not isinstance(password, basestring):
            raise TypeError('Expected property password to be a basestring')
        __self__.password = password
        """
        User's password.
        """
        __props__['password'] = password

        if not region:
            raise TypeError('Missing required property region')
        elif not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        Openstack region resource is created in.
        """
        __props__['region'] = region

        super(User, __self__).__init__(
            'openstack:database/user:User',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'databases' in outs:
            self.databases = outs['databases']
        if 'host' in outs:
            self.host = outs['host']
        if 'instanceId' in outs:
            self.instance_id = outs['instanceId']
        if 'name' in outs:
            self.name = outs['name']
        if 'password' in outs:
            self.password = outs['password']
        if 'region' in outs:
            self.region = outs['region']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class ContainerObject(pulumi.CustomResource):
    """
    Manages a V1 container object resource within OpenStack.
    """
    def __init__(__self__, __name__, __opts__=None, container_name=None, content=None, content_disposition=None, content_encoding=None, content_type=None, copy_from=None, delete_after=None, delete_at=None, detect_content_type=None, etag=None, metadata=None, name=None, object_manifest=None, region=None, source=None):
        """Create a ContainerObject resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if not container_name:
            raise TypeError('Missing required property container_name')
        elif not isinstance(container_name, basestring):
            raise TypeError('Expected property container_name to be a basestring')
        __self__.container_name = container_name
        """
        A unique (within an account) name for the container. 
        The container name must be from 1 to 256 characters long and can start
        with any character and contain any pattern. Character set must be UTF-8.
        The container name cannot contain a slash (/) character because this
        character delimits the container and object name. For example, the path
        /v1/account/www/pages specifies the www container, not the www/pages container.
        """
        __props__['containerName'] = container_name

        if content and not isinstance(content, basestring):
            raise TypeError('Expected property content to be a basestring')
        __self__.content = content
        """
        A string representing the content of the object. Conflicts with
        `source` and `copy_from`.
        """
        __props__['content'] = content

        if content_disposition and not isinstance(content_disposition, basestring):
            raise TypeError('Expected property content_disposition to be a basestring')
        __self__.content_disposition = content_disposition
        """
        A string which specifies the override behavior for 
        the browser. For example, this header might specify that the browser use a download
        program to save this file rather than show the file, which is the default.
        """
        __props__['contentDisposition'] = content_disposition

        if content_encoding and not isinstance(content_encoding, basestring):
            raise TypeError('Expected property content_encoding to be a basestring')
        __self__.content_encoding = content_encoding
        """
        A string representing the value of the Content-Encoding
        metadata.
        """
        __props__['contentEncoding'] = content_encoding

        if content_type and not isinstance(content_type, basestring):
            raise TypeError('Expected property content_type to be a basestring')
        __self__.content_type = content_type
        """
        A string which sets the MIME type for the object.
        """
        __props__['contentType'] = content_type

        if copy_from and not isinstance(copy_from, basestring):
            raise TypeError('Expected property copy_from to be a basestring')
        __self__.copy_from = copy_from
        """
        A string representing the name of an object 
        used to create the new object by copying the `copy_from` object. The value is in form
        {container}/{object}. You must UTF-8-encode and then URL-encode the names of the
        container and object before you include them in the header. Conflicts with `source` and
        `content`.
        """
        __props__['copyFrom'] = copy_from

        if delete_after and not isinstance(delete_after, int):
            raise TypeError('Expected property delete_after to be a int')
        __self__.delete_after = delete_after
        """
        An integer representing the number of seconds after which the
        system removes the object. Internally, the Object Storage system stores this value in
        the X-Delete-At metadata item.
        """
        __props__['deleteAfter'] = delete_after

        if delete_at and not isinstance(delete_at, basestring):
            raise TypeError('Expected property delete_at to be a basestring')
        __self__.delete_at = delete_at
        """
        An string representing the date when the system removes the object. 
        For example, "2015-08-26" is equivalent to Mon, Wed, 26 Aug 2015 00:00:00 GMT.
        """
        __props__['deleteAt'] = delete_at

        if detect_content_type and not isinstance(detect_content_type, bool):
            raise TypeError('Expected property detect_content_type to be a bool')
        __self__.detect_content_type = detect_content_type
        """
        If set to true, Object Storage guesses the content 
        type based on the file extension and ignores the value sent in the Content-Type
        header, if present.
        """
        __props__['detectContentType'] = detect_content_type

        if etag and not isinstance(etag, basestring):
            raise TypeError('Expected property etag to be a basestring')
        __self__.etag = etag
        """
        Used to trigger updates. The only meaningful value is ${md5(file("path/to/file"))}.
        """
        __props__['etag'] = etag

        if metadata and not isinstance(metadata, dict):
            raise TypeError('Expected property metadata to be a dict')
        __self__.metadata = metadata
        __props__['metadata'] = metadata

        if name and not isinstance(name, basestring):
            raise TypeError('Expected property name to be a basestring')
        __self__.name = name
        """
        A unique name for the object.
        """
        __props__['name'] = name

        if object_manifest and not isinstance(object_manifest, basestring):
            raise TypeError('Expected property object_manifest to be a basestring')
        __self__.object_manifest = object_manifest
        """
        A string set to specify that this is a dynamic large 
        object manifest object. The value is the container and object name prefix of the
        segment objects in the form container/prefix. You must UTF-8-encode and then
        URL-encode the names of the container and prefix before you include them in this
        header.
        """
        __props__['objectManifest'] = object_manifest

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to create the container. If
        omitted, the `region` argument of the provider is used. Changing this
        creates a new container.
        """
        __props__['region'] = region

        if source and not isinstance(source, basestring):
            raise TypeError('Expected property source to be a basestring')
        __self__.source = source
        """
        A string representing the local path of a file which will be used
        as the object's content. Conflicts with `source` and `copy_from`.
        """
        __props__['source'] = source

        __self__.content_length = pulumi.runtime.UNKNOWN
        """
        If the operation succeeds, this value is zero (0) or the 
        length of informational or error text in the response body.
        """
        __self__.date = pulumi.runtime.UNKNOWN
        """
        The date and time the system responded to the request, using the preferred 
        format of RFC 7231 as shown in this example Thu, 16 Jun 2016 15:10:38 GMT. The
        time is always in UTC.
        """
        __self__.last_modified = pulumi.runtime.UNKNOWN
        """
        The date and time when the object was last modified. The date and time 
        stamp format is ISO 8601:
        CCYY-MM-DDThh:mm:ss±hh:mm
        For example, 2015-08-27T09:49:58-05:00.
        The ±hh:mm value, if included, is the time zone as an offset from UTC. In the previous
        example, the offset value is -05:00.
        """
        __self__.trans_id = pulumi.runtime.UNKNOWN
        """
        A unique transaction ID for this request. Your service provider might 
        need this value if you report a problem.
        """

        super(ContainerObject, __self__).__init__(
            'openstack:objectstorage/containerObject:ContainerObject',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'containerName' in outs:
            self.container_name = outs['containerName']
        if 'content' in outs:
            self.content = outs['content']
        if 'contentDisposition' in outs:
            self.content_disposition = outs['contentDisposition']
        if 'contentEncoding' in outs:
            self.content_encoding = outs['contentEncoding']
        if 'contentLength' in outs:
            self.content_length = outs['contentLength']
        if 'contentType' in outs:
            self.content_type = outs['contentType']
        if 'copyFrom' in outs:
            self.copy_from = outs['copyFrom']
        if 'date' in outs:
            self.date = outs['date']
        if 'deleteAfter' in outs:
            self.delete_after = outs['deleteAfter']
        if 'deleteAt' in outs:
            self.delete_at = outs['deleteAt']
        if 'detectContentType' in outs:
            self.detect_content_type = outs['detectContentType']
        if 'etag' in outs:
            self.etag = outs['etag']
        if 'lastModified' in outs:
            self.last_modified = outs['lastModified']
        if 'metadata' in outs:
            self.metadata = outs['metadata']
        if 'name' in outs:
            self.name = outs['name']
        if 'objectManifest' in outs:
            self.object_manifest = outs['objectManifest']
        if 'region' in outs:
            self.region = outs['region']
        if 'source' in outs:
            self.source = outs['source']
        if 'transId' in outs:
            self.trans_id = outs['transId']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class GetSubnetResult(object):
    """
    A collection of values returned by getSubnet.
    """
    def __init__(__self__, allocation_pools=None, cidr=None, dns_nameservers=None, enable_dhcp=None, gateway_ip=None, host_routes=None, ip_version=None, ipv6_address_mode=None, ipv6_ra_mode=None, name=None, network_id=None, region=None, subnet_id=None, subnetpool_id=None, tenant_id=None):
        if allocation_pools and not isinstance(allocation_pools, list):
            raise TypeError('Expected argument allocation_pools to be a list')
        __self__.allocation_pools = allocation_pools
        """
        Allocation pools of the subnet.
        """
        if cidr and not isinstance(cidr, basestring):
            raise TypeError('Expected argument cidr to be a basestring')
        __self__.cidr = cidr
        if dns_nameservers and not isinstance(dns_nameservers, list):
            raise TypeError('Expected argument dns_nameservers to be a list')
        __self__.dns_nameservers = dns_nameservers
        """
        DNS Nameservers of the subnet.
        """
        if enable_dhcp and not isinstance(enable_dhcp, bool):
            raise TypeError('Expected argument enable_dhcp to be a bool')
        __self__.enable_dhcp = enable_dhcp
        """
        Whether the subnet has DHCP enabled or not.
        """
        if gateway_ip and not isinstance(gateway_ip, basestring):
            raise TypeError('Expected argument gateway_ip to be a basestring')
        __self__.gateway_ip = gateway_ip
        if host_routes and not isinstance(host_routes, list):
            raise TypeError('Expected argument host_routes to be a list')
        __self__.host_routes = host_routes
        """
        Host Routes of the subnet.
        """
        if ip_version and not isinstance(ip_version, int):
            raise TypeError('Expected argument ip_version to be a int')
        __self__.ip_version = ip_version
        if ipv6_address_mode and not isinstance(ipv6_address_mode, basestring):
            raise TypeError('Expected argument ipv6_address_mode to be a basestring')
        __self__.ipv6_address_mode = ipv6_address_mode
        if ipv6_ra_mode and not isinstance(ipv6_ra_mode, basestring):
            raise TypeError('Expected argument ipv6_ra_mode to be a basestring')
        __self__.ipv6_ra_mode = ipv6_ra_mode
        if name and not isinstance(name, basestring):
            raise TypeError('Expected argument name to be a basestring')
        __self__.name = name
        if network_id and not isinstance(network_id, basestring):
            raise TypeError('Expected argument network_id to be a basestring')
        __self__.network_id = network_id
        if region and not isinstance(region, basestring):
            raise TypeError('Expected argument region to be a basestring')
        __self__.region = region
        """
        See Argument Reference above.
        """
        if subnet_id and not isinstance(subnet_id, basestring):
            raise TypeError('Expected argument subnet_id to be a basestring')
        __self__.subnet_id = subnet_id
        if subnetpool_id and not isinstance(subnetpool_id, basestring):
            raise TypeError('Expected argument subnetpool_id to be a basestring')
        __self__.subnetpool_id = subnetpool_id
        if tenant_id and not isinstance(tenant_id, basestring):
            raise TypeError('Expected argument tenant_id to be a basestring')
        __self__.tenant_id = tenant_id

def get_subnet(cidr=None, dhcp_disabled=None, dhcp_enabled=None, gateway_ip=None, ip_version=None, ipv6_address_mode=None, ipv6_ra_mode=None, name=None, network_id=None, region=None, subnet_id=None, subnetpool_id=None, tenant_id=None):
    """
    Use this data source to get the ID of an available OpenStack subnet.
    """
    __args__ = dict()

    __args__['cidr'] = cidr
    __args__['dhcpDisabled'] = dhcp_disabled
    __args__['dhcpEnabled'] = dhcp_enabled
    __args__['gatewayIp'] = gateway_ip
    __args__['ipVersion'] = ip_version
    __args__['ipv6AddressMode'] = ipv6_address_mode
    __args__['ipv6RaMode'] = ipv6_ra_mode
    __args__['name'] = name
    __args__['networkId'] = network_id
    __args__['region'] = region
    __args__['subnetId'] = subnet_id
    __args__['subnetpoolId'] = subnetpool_id
    __args__['tenantId'] = tenant_id
    __ret__ = pulumi.runtime.invoke('openstack:networking/getSubnet:getSubnet', __args__)

    return GetSubnetResult(
        allocation_pools=__ret__.get('allocationPools'),
        cidr=__ret__.get('cidr'),
        dns_nameservers=__ret__.get('dnsNameservers'),
        enable_dhcp=__ret__.get('enableDhcp'),
        gateway_ip=__ret__.get('gatewayIp'),
        host_routes=__ret__.get('hostRoutes'),
        ip_version=__ret__.get('ipVersion'),
        ipv6_address_mode=__ret__.get('ipv6AddressMode'),
        ipv6_ra_mode=__ret__.get('ipv6RaMode'),
        name=__ret__.get('name'),
        network_id=__ret__.get('networkId'),
        region=__ret__.get('region'),
        subnet_id=__ret__.get('subnetId'),
        subnetpool_id=__ret__.get('subnetpoolId'),
        tenant_id=__ret__.get('tenantId'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class GetNetworkResult(object):
    """
    A collection of values returned by getNetwork.
    """
    def __init__(__self__, admin_state_up=None, availability_zone_hints=None, region=None, shared=None):
        if admin_state_up and not isinstance(admin_state_up, basestring):
            raise TypeError('Expected argument admin_state_up to be a basestring')
        __self__.admin_state_up = admin_state_up
        """
        (Optional) The administrative state of the network.
        """
        if availability_zone_hints and not isinstance(availability_zone_hints, list):
            raise TypeError('Expected argument availability_zone_hints to be a list')
        __self__.availability_zone_hints = availability_zone_hints
        """
        (Optional) The availability zone candidates for the network.
        """
        if region and not isinstance(region, basestring):
            raise TypeError('Expected argument region to be a basestring')
        __self__.region = region
        """
        See Argument Reference above.
        """
        if shared and not isinstance(shared, basestring):
            raise TypeError('Expected argument shared to be a basestring')
        __self__.shared = shared
        """
        (Optional)  Specifies whether the network resource can be accessed
        by any tenant or not.
        """

def get_network(matching_subnet_cidr=None, name=None, network_id=None, region=None, status=None, tenant_id=None):
    """
    Use this data source to get the ID of an available OpenStack network.
    """
    __args__ = dict()

    __args__['matchingSubnetCidr'] = matching_subnet_cidr
    __args__['name'] = name
    __args__['networkId'] = network_id
    __args__['region'] = region
    __args__['status'] = status
    __args__['tenantId'] = tenant_id
    __ret__ = pulumi.runtime.invoke('openstack:networking/getNetwork:getNetwork', __args__)

    return GetNetworkResult(
        admin_state_up=__ret__.get('adminStateUp'),
        availability_zone_hints=__ret__.get('availabilityZoneHints'),
        region=__ret__.get('region'),
        shared=__ret__.get('shared'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class VolumeAttachV2(pulumi.CustomResource):
    """
    This resource is experimental and may be removed in the future! Feedback
    is requested if you find this resource useful or if you find any problems
    with it.
    
    Creates a general purpose attachment connection to a Block
    Storage volume using the OpenStack Block Storage (Cinder) v2 API.
    Depending on your Block Storage service configuration, this
    resource can assist in attaching a volume to a non-OpenStack resource
    such as a bare-metal server or a remote virtual machine in a
    different cloud provider.
    
    This does not actually attach a volume to an instance. Please use
    the `openstack_compute_volume_attach_v2` resource for that.
    """
    def __init__(__self__, __name__, __opts__=None, attach_mode=None, device=None, host_name=None, initiator=None, ip_address=None, multipath=None, os_type=None, platform=None, region=None, volume_id=None, wwnn=None, wwpns=None):
        """Create a VolumeAttachV2 resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if attach_mode and not isinstance(attach_mode, basestring):
            raise TypeError('Expected property attach_mode to be a basestring')
        __self__.attach_mode = attach_mode
        """
        Specify whether to attach the volume as Read-Only
        (`ro`) or Read-Write (`rw`). Only values of `ro` and `rw` are accepted.
        If left unspecified, the Block Storage API will apply a default of `rw`.
        """
        __props__['attachMode'] = attach_mode

        if device and not isinstance(device, basestring):
            raise TypeError('Expected property device to be a basestring')
        __self__.device = device
        """
        The device to tell the Block Storage service this
        volume will be attached as. This is purely for informational purposes.
        You can specify `auto` or a device such as `/dev/vdc`.
        """
        __props__['device'] = device

        if not host_name:
            raise TypeError('Missing required property host_name')
        elif not isinstance(host_name, basestring):
            raise TypeError('Expected property host_name to be a basestring')
        __self__.host_name = host_name
        """
        The host to attach the volume to.
        """
        __props__['hostName'] = host_name

        if initiator and not isinstance(initiator, basestring):
            raise TypeError('Expected property initiator to be a basestring')
        __self__.initiator = initiator
        """
        The iSCSI initiator string to make the connection.
        """
        __props__['initiator'] = initiator

        if ip_address and not isinstance(ip_address, basestring):
            raise TypeError('Expected property ip_address to be a basestring')
        __self__.ip_address = ip_address
        """
        The IP address of the `host_name` above.
        """
        __props__['ipAddress'] = ip_address

        if multipath and not isinstance(multipath, bool):
            raise TypeError('Expected property multipath to be a bool')
        __self__.multipath = multipath
        """
        Whether to connect to this volume via multipath.
        """
        __props__['multipath'] = multipath

        if os_type and not isinstance(os_type, basestring):
            raise TypeError('Expected property os_type to be a basestring')
        __self__.os_type = os_type
        """
        The iSCSI initiator OS type.
        """
        __props__['osType'] = os_type

        if platform and not isinstance(platform, basestring):
            raise TypeError('Expected property platform to be a basestring')
        __self__.platform = platform
        """
        The iSCSI initiator platform.
        """
        __props__['platform'] = platform

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to obtain the V2 Block Storage
        client. A Block Storage client is needed to create a volume attachment.
        If omitted, the `region` argument of the provider is used. Changing this
        creates a new volume attachment.
        """
        __props__['region'] = region

        if not volume_id:
            raise TypeError('Missing required property volume_id')
        elif not isinstance(volume_id, basestring):
            raise TypeError('Expected property volume_id to be a basestring')
        __self__.volume_id = volume_id
        """
        The ID of the Volume to attach to an Instance.
        """
        __props__['volumeId'] = volume_id

        if wwnn and not isinstance(wwnn, basestring):
            raise TypeError('Expected property wwnn to be a basestring')
        __self__.wwnn = wwnn
        """
        A wwnn name. Used for Fibre Channel connections.
        """
        __props__['wwnn'] = wwnn

        if wwpns and not isinstance(wwpns, list):
            raise TypeError('Expected property wwpns to be a list')
        __self__.wwpns = wwpns
        """
        An array of wwpn strings. Used for Fibre Channel
        connections.
        """
        __props__['wwpns'] = wwpns

        __self__.data = pulumi.runtime.UNKNOWN
        """
        This is a map of key/value pairs that contain the connection
        information. You will want to pass this information to a provisioner
        script to finalize the connection. See below for more information.
        """
        __self__.driver_volume_type = pulumi.runtime.UNKNOWN
        """
        The storage driver that the volume is based on.
        """
        __self__.mount_point_base = pulumi.runtime.UNKNOWN
        """
        A mount point base name for shared storage.
        """

        super(VolumeAttachV2, __self__).__init__(
            'openstack:blockstorage/volumeAttachV2:VolumeAttachV2',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'attachMode' in outs:
            self.attach_mode = outs['attachMode']
        if 'data' in outs:
            self.data = outs['data']
        if 'device' in outs:
            self.device = outs['device']
        if 'driverVolumeType' in outs:
            self.driver_volume_type = outs['driverVolumeType']
        if 'hostName' in outs:
            self.host_name = outs['hostName']
        if 'initiator' in outs:
            self.initiator = outs['initiator']
        if 'ipAddress' in outs:
            self.ip_address = outs['ipAddress']
        if 'mountPointBase' in outs:
            self.mount_point_base = outs['mountPointBase']
        if 'multipath' in outs:
            self.multipath = outs['multipath']
        if 'osType' in outs:
            self.os_type = outs['osType']
        if 'platform' in outs:
            self.platform = outs['platform']
        if 'region' in outs:
            self.region = outs['region']
        if 'volumeId' in outs:
            self.volume_id = outs['volumeId']
        if 'wwnn' in outs:
            self.wwnn = outs['wwnn']
        if 'wwpns' in outs:
            self.wwpns = outs['wwpns']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class GetAuthScopeResult(object):
    """
    A collection of values returned by getAuthScope.
    """
    def __init__(__self__, project_domain_id=None, project_domain_name=None, project_id=None, project_name=None, region=None, roles=None, user_domain_id=None, user_domain_name=None, user_id=None, user_name=None):
        if project_domain_id and not isinstance(project_domain_id, basestring):
            raise TypeError('Expected argument project_domain_id to be a basestring')
        __self__.project_domain_id = project_domain_id
        """
        The domain ID of the project.
        """
        if project_domain_name and not isinstance(project_domain_name, basestring):
            raise TypeError('Expected argument project_domain_name to be a basestring')
        __self__.project_domain_name = project_domain_name
        """
        The domain name of the project.
        """
        if project_id and not isinstance(project_id, basestring):
            raise TypeError('Expected argument project_id to be a basestring')
        __self__.project_id = project_id
        """
        The project ID of the scope.
        """
        if project_name and not isinstance(project_name, basestring):
            raise TypeError('Expected argument project_name to be a basestring')
        __self__.project_name = project_name
        """
        The project name of the scope.
        """
        if region and not isinstance(region, basestring):
            raise TypeError('Expected argument region to be a basestring')
        __self__.region = region
        if roles and not isinstance(roles, list):
            raise TypeError('Expected argument roles to be a list')
        __self__.roles = roles
        """
        A list of roles in the current scope. See reference below.
        """
        if user_domain_id and not isinstance(user_domain_id, basestring):
            raise TypeError('Expected argument user_domain_id to be a basestring')
        __self__.user_domain_id = user_domain_id
        """
        The domain ID of the user.
        """
        if user_domain_name and not isinstance(user_domain_name, basestring):
            raise TypeError('Expected argument user_domain_name to be a basestring')
        __self__.user_domain_name = user_domain_name
        """
        The domain name of the user.
        """
        if user_id and not isinstance(user_id, basestring):
            raise TypeError('Expected argument user_id to be a basestring')
        __self__.user_id = user_id
        """
        The user ID the of the scope.
        """
        if user_name and not isinstance(user_name, basestring):
            raise TypeError('Expected argument user_name to be a basestring')
        __self__.user_name = user_name
        """
        The username of the scope.
        """

def get_auth_scope(name=None, region=None):
    """
    Use this data source to get authentication information about the current
    auth scope in use. This can be used as self-discovery or introspection of
    the username or project name currently in use.
    """
    __args__ = dict()

    __args__['name'] = name
    __args__['region'] = region
    __ret__ = pulumi.runtime.invoke('openstack:identity/getAuthScope:getAuthScope', __args__)

    return GetAuthScopeResult(
        project_domain_id=__ret__.get('projectDomainId'),
        project_domain_name=__ret__.get('projectDomainName'),
        project_id=__ret__.get('projectId'),
        project_name=__ret__.get('projectName'),
        region=__ret__.get('region'),
        roles=__ret__.get('roles'),
        user_domain_id=__ret__.get('userDomainId'),
        user_domain_name=__ret__.get('userDomainName'),
        user_id=__ret__.get('userId'),
        user_name=__ret__.get('userName'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class GetImageResult(object):
    """
    A collection of values returned by getImage.
    """
    def __init__(__self__, checksum=None, container_format=None, disk_format=None, file=None, metadata=None, min_disk_gb=None, min_ram_mb=None, protected=None, region=None, schema=None, size_bytes=None, updated_at=None):
        if checksum and not isinstance(checksum, basestring):
            raise TypeError('Expected argument checksum to be a basestring')
        __self__.checksum = checksum
        """
        The checksum of the data associated with the image.
        """
        if container_format and not isinstance(container_format, basestring):
            raise TypeError('Expected argument container_format to be a basestring')
        __self__.container_format = container_format
        if disk_format and not isinstance(disk_format, basestring):
            raise TypeError('Expected argument disk_format to be a basestring')
        __self__.disk_format = disk_format
        if file and not isinstance(file, basestring):
            raise TypeError('Expected argument file to be a basestring')
        __self__.file = file
        """
        the trailing path after the glance endpoint that represent the
        location of the image or the path to retrieve it.
        """
        if metadata and not isinstance(metadata, dict):
            raise TypeError('Expected argument metadata to be a dict')
        __self__.metadata = metadata
        """
        The metadata associated with the image.
        Image metadata allow for meaningfully define the image properties
        and tags. See http://docs.openstack.org/developer/glance/metadefs-concepts.html.
        """
        if min_disk_gb and not isinstance(min_disk_gb, int):
            raise TypeError('Expected argument min_disk_gb to be a int')
        __self__.min_disk_gb = min_disk_gb
        """
        The minimum amount of disk space required to use the image.
        """
        if min_ram_mb and not isinstance(min_ram_mb, int):
            raise TypeError('Expected argument min_ram_mb to be a int')
        __self__.min_ram_mb = min_ram_mb
        """
        The minimum amount of ram required to use the image.
        """
        if protected and not isinstance(protected, bool):
            raise TypeError('Expected argument protected to be a bool')
        __self__.protected = protected
        """
        Whether or not the image is protected.
        """
        if region and not isinstance(region, basestring):
            raise TypeError('Expected argument region to be a basestring')
        __self__.region = region
        if schema and not isinstance(schema, basestring):
            raise TypeError('Expected argument schema to be a basestring')
        __self__.schema = schema
        """
        The path to the JSON-schema that represent
        the image or image
        """
        if size_bytes and not isinstance(size_bytes, int):
            raise TypeError('Expected argument size_bytes to be a int')
        __self__.size_bytes = size_bytes
        """
        The size of the image (in bytes).
        """
        if updated_at and not isinstance(updated_at, basestring):
            raise TypeError('Expected argument updated_at to be a basestring')
        __self__.updated_at = updated_at

def get_image(member_status=None, most_recent=None, name=None, owner=None, properties=None, region=None, size_max=None, size_min=None, sort_direction=None, sort_key=None, tag=None, visibility=None):
    """
    Use this data source to get the ID of an available OpenStack image.
    """
    __args__ = dict()

    __args__['memberStatus'] = member_status
    __args__['mostRecent'] = most_recent
    __args__['name'] = name
    __args__['owner'] = owner
    __args__['properties'] = properties
    __args__['region'] = region
    __args__['sizeMax'] = size_max
    __args__['sizeMin'] = size_min
    __args__['sortDirection'] = sort_direction
    __args__['sortKey'] = sort_key
    __args__['tag'] = tag
    __args__['visibility'] = visibility
    __ret__ = pulumi.runtime.invoke('openstack:images/getImage:getImage', __args__)

    return GetImageResult(
        checksum=__ret__.get('checksum'),
        container_format=__ret__.get('containerFormat'),
        disk_format=__ret__.get('diskFormat'),
        file=__ret__.get('file'),
        metadata=__ret__.get('metadata'),
        min_disk_gb=__ret__.get('minDiskGb'),
        min_ram_mb=__ret__.get('minRamMb'),
        protected=__ret__.get('protected'),
        region=__ret__.get('region'),
        schema=__ret__.get('schema'),
        size_bytes=__ret__.get('sizeBytes'),
        updated_at=__ret__.get('updatedAt'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class FloatingIpAssociate(pulumi.CustomResource):
    """
    Associate a floating IP to an instance. This can be used instead of the
    `floating_ip` options in `openstack_compute_instance_v2`.
    """
    def __init__(__self__, __name__, __opts__=None, fixed_ip=None, floating_ip=None, instance_id=None, region=None, wait_until_associated=None):
        """Create a FloatingIpAssociate resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if fixed_ip and not isinstance(fixed_ip, basestring):
            raise TypeError('Expected property fixed_ip to be a basestring')
        __self__.fixed_ip = fixed_ip
        """
        The specific IP address to direct traffic to.
        """
        __props__['fixedIp'] = fixed_ip

        if not floating_ip:
            raise TypeError('Missing required property floating_ip')
        elif not isinstance(floating_ip, basestring):
            raise TypeError('Expected property floating_ip to be a basestring')
        __self__.floating_ip = floating_ip
        """
        The floating IP to associate.
        """
        __props__['floatingIp'] = floating_ip

        if not instance_id:
            raise TypeError('Missing required property instance_id')
        elif not isinstance(instance_id, basestring):
            raise TypeError('Expected property instance_id to be a basestring')
        __self__.instance_id = instance_id
        """
        The instance to associte the floating IP with.
        """
        __props__['instanceId'] = instance_id

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to obtain the V2 Compute client.
        Keypairs are associated with accounts, but a Compute client is needed to
        create one. If omitted, the `region` argument of the provider is used.
        Changing this creates a new floatingip_associate.
        """
        __props__['region'] = region

        if wait_until_associated and not isinstance(wait_until_associated, bool):
            raise TypeError('Expected property wait_until_associated to be a bool')
        __self__.wait_until_associated = wait_until_associated
        """
        In cases where the OpenStack environment
        does not automatically wait until the association has finished, set this
        option to have Terraform poll the instance until the floating IP has been
        associated. Defaults to false.
        """
        __props__['waitUntilAssociated'] = wait_until_associated

        super(FloatingIpAssociate, __self__).__init__(
            'openstack:compute/floatingIpAssociate:FloatingIpAssociate',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'fixedIp' in outs:
            self.fixed_ip = outs['fixedIp']
        if 'floatingIp' in outs:
            self.floating_ip = outs['floatingIp']
        if 'instanceId' in outs:
            self.instance_id = outs['instanceId']
        if 'region' in outs:
            self.region = outs['region']
        if 'waitUntilAssociated' in outs:
            self.wait_until_associated = outs['waitUntilAssociated']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class MemberV1(pulumi.CustomResource):
    """
    Manages a V1 load balancer member resource within OpenStack.
    """
    def __init__(__self__, __name__, __opts__=None, address=None, admin_state_up=None, pool_id=None, port=None, region=None, tenant_id=None, weight=None):
        """Create a MemberV1 resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if not address:
            raise TypeError('Missing required property address')
        elif not isinstance(address, basestring):
            raise TypeError('Expected property address to be a basestring')
        __self__.address = address
        """
        The IP address of the member. Changing this creates a
        new member.
        """
        __props__['address'] = address

        if admin_state_up and not isinstance(admin_state_up, bool):
            raise TypeError('Expected property admin_state_up to be a bool')
        __self__.admin_state_up = admin_state_up
        """
        The administrative state of the member.
        Acceptable values are 'true' and 'false'. Changing this value updates the
        state of the existing member.
        """
        __props__['adminStateUp'] = admin_state_up

        if not pool_id:
            raise TypeError('Missing required property pool_id')
        elif not isinstance(pool_id, basestring):
            raise TypeError('Expected property pool_id to be a basestring')
        __self__.pool_id = pool_id
        """
        The ID of the LB pool. Changing this creates a new
        member.
        """
        __props__['poolId'] = pool_id

        if not port:
            raise TypeError('Missing required property port')
        elif not isinstance(port, int):
            raise TypeError('Expected property port to be a int')
        __self__.port = port
        """
        An integer representing the port on which the member is
        hosted. Changing this creates a new member.
        """
        __props__['port'] = port

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to obtain the V2 Networking client.
        A Networking client is needed to create an LB member. If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        LB member.
        """
        __props__['region'] = region

        if tenant_id and not isinstance(tenant_id, basestring):
            raise TypeError('Expected property tenant_id to be a basestring')
        __self__.tenant_id = tenant_id
        """
        The owner of the member. Required if admin wants to
        create a member for another tenant. Changing this creates a new member.
        """
        __props__['tenantId'] = tenant_id

        if weight and not isinstance(weight, int):
            raise TypeError('Expected property weight to be a int')
        __self__.weight = weight
        """
        The load balancing weight of the member. This is currently unable
        to be set through Terraform.
        """
        __props__['weight'] = weight

        super(MemberV1, __self__).__init__(
            'openstack:loadbalancer/memberV1:MemberV1',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'address' in outs:
            self.address = outs['address']
        if 'adminStateUp' in outs:
            self.admin_state_up = outs['adminStateUp']
        if 'poolId' in outs:
            self.pool_id = outs['poolId']
        if 'port' in outs:
            self.port = outs['port']
        if 'region' in outs:
            self.region = outs['region']
        if 'tenantId' in outs:
            self.tenant_id = outs['tenantId']
        if 'weight' in outs:
            self.weight = outs['weight']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class RouterRoute(pulumi.CustomResource):
    """
    Creates a routing entry on a OpenStack V2 router.
    """
    def __init__(__self__, __name__, __opts__=None, destination_cidr=None, next_hop=None, region=None, router_id=None):
        """Create a RouterRoute resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if not destination_cidr:
            raise TypeError('Missing required property destination_cidr')
        elif not isinstance(destination_cidr, basestring):
            raise TypeError('Expected property destination_cidr to be a basestring')
        __self__.destination_cidr = destination_cidr
        """
        CIDR block to match on the packet’s destination IP. Changing
        this creates a new routing entry.
        """
        __props__['destinationCidr'] = destination_cidr

        if not next_hop:
            raise TypeError('Missing required property next_hop')
        elif not isinstance(next_hop, basestring):
            raise TypeError('Expected property next_hop to be a basestring')
        __self__.next_hop = next_hop
        """
        IP address of the next hop gateway.  Changing
        this creates a new routing entry.
        """
        __props__['nextHop'] = next_hop

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to obtain the V2 networking client.
        A networking client is needed to configure a routing entry on a router. If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        routing entry.
        """
        __props__['region'] = region

        if not router_id:
            raise TypeError('Missing required property router_id')
        elif not isinstance(router_id, basestring):
            raise TypeError('Expected property router_id to be a basestring')
        __self__.router_id = router_id
        """
        ID of the router this routing entry belongs to. Changing
        this creates a new routing entry.
        """
        __props__['routerId'] = router_id

        super(RouterRoute, __self__).__init__(
            'openstack:networking/routerRoute:RouterRoute',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'destinationCidr' in outs:
            self.destination_cidr = outs['destinationCidr']
        if 'nextHop' in outs:
            self.next_hop = outs['nextHop']
        if 'region' in outs:
            self.region = outs['region']
        if 'routerId' in outs:
            self.router_id = outs['routerId']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class Instance(pulumi.CustomResource):
    """
    Manages a V1 DB instance resource within OpenStack.
    """
    def __init__(__self__, __name__, __opts__=None, configuration_id=None, databases=None, datastore=None, flavor_id=None, name=None, networks=None, region=None, size=None, users=None):
        """Create a Instance resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if configuration_id and not isinstance(configuration_id, basestring):
            raise TypeError('Expected property configuration_id to be a basestring')
        __self__.configuration_id = configuration_id
        """
        Configuration ID to be attached to the instance. Database instance
        will be rebooted when configuration is detached.
        """
        __props__['configurationId'] = configuration_id

        if databases and not isinstance(databases, list):
            raise TypeError('Expected property databases to be a list')
        __self__.databases = databases
        """
        An array of database name, charset and collate. The database
        object structure is documented below.
        """
        __props__['databases'] = databases

        if not datastore:
            raise TypeError('Missing required property datastore')
        elif not isinstance(datastore, dict):
            raise TypeError('Expected property datastore to be a dict')
        __self__.datastore = datastore
        """
        An array of database engine type and version. The datastore
        object structure is documented below. Changing this creates a new instance.
        """
        __props__['datastore'] = datastore

        if flavor_id and not isinstance(flavor_id, basestring):
            raise TypeError('Expected property flavor_id to be a basestring')
        __self__.flavor_id = flavor_id
        """
        The flavor ID of the desired flavor for the instance.
        Changing this creates new instance.
        """
        __props__['flavorId'] = flavor_id

        if name and not isinstance(name, basestring):
            raise TypeError('Expected property name to be a basestring')
        __self__.name = name
        """
        Database to be created on new instance. Changing this creates a
        new instance.
        """
        __props__['name'] = name

        if networks and not isinstance(networks, list):
            raise TypeError('Expected property networks to be a list')
        __self__.networks = networks
        """
        An array of one or more networks to attach to the
        instance. The network object structure is documented below. Changing this
        creates a new instance.
        """
        __props__['networks'] = networks

        if not region:
            raise TypeError('Missing required property region')
        elif not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to create the db instance. Changing this
        creates a new instance.
        """
        __props__['region'] = region

        if not size:
            raise TypeError('Missing required property size')
        elif not isinstance(size, int):
            raise TypeError('Expected property size to be a int')
        __self__.size = size
        """
        Specifies the volume size in GB. Changing this creates new instance.
        """
        __props__['size'] = size

        if users and not isinstance(users, list):
            raise TypeError('Expected property users to be a list')
        __self__.users = users
        """
        An array of username, password, host and databases. The user
        object structure is documented below.
        """
        __props__['users'] = users

        super(Instance, __self__).__init__(
            'openstack:database/instance:Instance',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'configurationId' in outs:
            self.configuration_id = outs['configurationId']
        if 'databases' in outs:
            self.databases = outs['databases']
        if 'datastore' in outs:
            self.datastore = outs['datastore']
        if 'flavorId' in outs:
            self.flavor_id = outs['flavorId']
        if 'name' in outs:
            self.name = outs['name']
        if 'networks' in outs:
            self.networks = outs['networks']
        if 'region' in outs:
            self.region = outs['region']
        if 'size' in outs:
            self.size = outs['size']
        if 'users' in outs:
            self.users = outs['users']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class Instance(pulumi.CustomResource):
    """
    Manages a V2 VM instance resource within OpenStack.
    """
    def __init__(__self__, __name__, __opts__=None, access_ip_v4=None, access_ip_v6=None, admin_pass=None, availability_zone=None, block_devices=None, config_drive=None, flavor_id=None, flavor_name=None, force_delete=None, image_id=None, image_name=None, key_pair=None, metadata=None, name=None, networks=None, personalities=None, power_state=None, region=None, scheduler_hints=None, security_groups=None, stop_before_destroy=None, user_data=None):
        """Create a Instance resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if access_ip_v4 and not isinstance(access_ip_v4, basestring):
            raise TypeError('Expected property access_ip_v4 to be a basestring')
        __self__.access_ip_v4 = access_ip_v4
        """
        The first detected Fixed IPv4 address _or_ the
        Floating IP.
        """
        __props__['accessIpV4'] = access_ip_v4

        if access_ip_v6 and not isinstance(access_ip_v6, basestring):
            raise TypeError('Expected property access_ip_v6 to be a basestring')
        __self__.access_ip_v6 = access_ip_v6
        """
        The first detected Fixed IPv6 address.
        """
        __props__['accessIpV6'] = access_ip_v6

        if admin_pass and not isinstance(admin_pass, basestring):
            raise TypeError('Expected property admin_pass to be a basestring')
        __self__.admin_pass = admin_pass
        """
        The administrative password to assign to the server.
        Changing this changes the root password on the existing server.
        """
        __props__['adminPass'] = admin_pass

        if availability_zone and not isinstance(availability_zone, basestring):
            raise TypeError('Expected property availability_zone to be a basestring')
        __self__.availability_zone = availability_zone
        """
        The availability zone in which to create
        the server. Changing this creates a new server.
        """
        __props__['availabilityZone'] = availability_zone

        if block_devices and not isinstance(block_devices, list):
            raise TypeError('Expected property block_devices to be a list')
        __self__.block_devices = block_devices
        """
        Configuration of block devices. The block_device
        structure is documented below. Changing this creates a new server.
        You can specify multiple block devices which will create an instance with
        multiple disks. This configuration is very flexible, so please see the
        following [reference](http://docs.openstack.org/developer/nova/block_device_mapping.html)
        for more information.
        """
        __props__['blockDevices'] = block_devices

        if config_drive and not isinstance(config_drive, bool):
            raise TypeError('Expected property config_drive to be a bool')
        __self__.config_drive = config_drive
        """
        Whether to use the config_drive feature to
        configure the instance. Changing this creates a new server.
        """
        __props__['configDrive'] = config_drive

        if flavor_id and not isinstance(flavor_id, basestring):
            raise TypeError('Expected property flavor_id to be a basestring')
        __self__.flavor_id = flavor_id
        """
        The flavor ID of
        the desired flavor for the server. Changing this resizes the existing server.
        """
        __props__['flavorId'] = flavor_id

        if flavor_name and not isinstance(flavor_name, basestring):
            raise TypeError('Expected property flavor_name to be a basestring')
        __self__.flavor_name = flavor_name
        """
        The name of the
        desired flavor for the server. Changing this resizes the existing server.
        """
        __props__['flavorName'] = flavor_name

        if force_delete and not isinstance(force_delete, bool):
            raise TypeError('Expected property force_delete to be a bool')
        __self__.force_delete = force_delete
        """
        Whether to force the OpenStack instance to be
        forcefully deleted. This is useful for environments that have reclaim / soft
        deletion enabled.
        """
        __props__['forceDelete'] = force_delete

        if image_id and not isinstance(image_id, basestring):
            raise TypeError('Expected property image_id to be a basestring')
        __self__.image_id = image_id
        """
        (Optional; Required if `image_name` is empty and not booting
        from a volume. Do not specify if booting from a volume.) The image ID of
        the desired image for the server. Changing this creates a new server.
        """
        __props__['imageId'] = image_id

        if image_name and not isinstance(image_name, basestring):
            raise TypeError('Expected property image_name to be a basestring')
        __self__.image_name = image_name
        """
        (Optional; Required if `image_id` is empty and not booting
        from a volume. Do not specify if booting from a volume.) The name of the
        desired image for the server. Changing this creates a new server.
        """
        __props__['imageName'] = image_name

        if key_pair and not isinstance(key_pair, basestring):
            raise TypeError('Expected property key_pair to be a basestring')
        __self__.key_pair = key_pair
        """
        The name of a key pair to put on the server. The key
        pair must already be created and associated with the tenant's account.
        Changing this creates a new server.
        """
        __props__['keyPair'] = key_pair

        if metadata and not isinstance(metadata, dict):
            raise TypeError('Expected property metadata to be a dict')
        __self__.metadata = metadata
        """
        Metadata key/value pairs to make available from
        within the instance. Changing this updates the existing server metadata.
        """
        __props__['metadata'] = metadata

        if name and not isinstance(name, basestring):
            raise TypeError('Expected property name to be a basestring')
        __self__.name = name
        """
        The human-readable
        name of the network. Changing this creates a new server.
        """
        __props__['name'] = name

        if networks and not isinstance(networks, list):
            raise TypeError('Expected property networks to be a list')
        __self__.networks = networks
        """
        An array of one or more networks to attach to the
        instance. The network object structure is documented below. Changing this
        creates a new server.
        """
        __props__['networks'] = networks

        if personalities and not isinstance(personalities, list):
            raise TypeError('Expected property personalities to be a list')
        __self__.personalities = personalities
        """
        Customize the personality of an instance by
        defining one or more files and their contents. The personality structure
        is described below.
        """
        __props__['personalities'] = personalities

        if power_state and not isinstance(power_state, basestring):
            raise TypeError('Expected property power_state to be a basestring')
        __self__.power_state = power_state
        """
        Provide the VM state. Only 'active' and 'shutoff'
        are supported values. *Note*: If the initial power_state is the shutoff
        the VM will be stopped immediately after build and the provisioners like
        remote-exec or files are not supported.
        """
        __props__['powerState'] = power_state

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to create the server instance. If
        omitted, the `region` argument of the provider is used. Changing this
        creates a new server.
        """
        __props__['region'] = region

        if scheduler_hints and not isinstance(scheduler_hints, list):
            raise TypeError('Expected property scheduler_hints to be a list')
        __self__.scheduler_hints = scheduler_hints
        """
        Provide the Nova scheduler with hints on how
        the instance should be launched. The available hints are described below.
        """
        __props__['schedulerHints'] = scheduler_hints

        if security_groups and not isinstance(security_groups, list):
            raise TypeError('Expected property security_groups to be a list')
        __self__.security_groups = security_groups
        """
        An array of one or more security group names
        to associate with the server. Changing this results in adding/removing
        security groups from the existing server. *Note*: When attaching the
        instance to networks using Ports, place the security groups on the Port
        and not the instance.
        """
        __props__['securityGroups'] = security_groups

        if stop_before_destroy and not isinstance(stop_before_destroy, bool):
            raise TypeError('Expected property stop_before_destroy to be a bool')
        __self__.stop_before_destroy = stop_before_destroy
        """
        Whether to try stop instance gracefully
        before destroying it, thus giving chance for guest OS daemons to stop correctly.
        If instance doesn't stop within timeout, it will be destroyed anyway.
        """
        __props__['stopBeforeDestroy'] = stop_before_destroy

        if user_data and not isinstance(user_data, basestring):
            raise TypeError('Expected property user_data to be a basestring')
        __self__.user_data = user_data
        """
        The user data to provide when launching the instance.
        Changing this creates a new server.
        """
        __props__['userData'] = user_data

        __self__.all_metadata = pulumi.runtime.UNKNOWN
        """
        Contains all instance metadata, even metadata not set
        by Terraform.
        """

        super(Instance, __self__).__init__(
            'openstack:compute/instance:Instance',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'accessIpV4' in outs:
            self.access_ip_v4 = outs['accessIpV4']
        if 'accessIpV6' in outs:
            self.access_ip_v6 = outs['accessIpV6']
        if 'adminPass' in outs:
            self.admin_pass = outs['adminPass']
        if 'allMetadata' in outs:
            self.all_metadata = outs['allMetadata']
        if 'availabilityZone' in outs:
            self.availability_zone = outs['availabilityZone']
        if 'blockDevices' in outs:
            self.block_devices = outs['blockDevices']
        if 'configDrive' in outs:
            self.config_drive = outs['configDrive']
        if 'flavorId' in outs:
            self.flavor_id = outs['flavorId']
        if 'flavorName' in outs:
            self.flavor_name = outs['flavorName']
        if 'forceDelete' in outs:
            self.force_delete = outs['forceDelete']
        if 'imageId' in outs:
            self.image_id = outs['imageId']
        if 'imageName' in outs:
            self.image_name = outs['imageName']
        if 'keyPair' in outs:
            self.key_pair = outs['keyPair']
        if 'metadata' in outs:
            self.metadata = outs['metadata']
        if 'name' in outs:
            self.name = outs['name']
        if 'networks' in outs:
            self.networks = outs['networks']
        if 'personalities' in outs:
            self.personalities = outs['personalities']
        if 'powerState' in outs:
            self.power_state = outs['powerState']
        if 'region' in outs:
            self.region = outs['region']
        if 'schedulerHints' in outs:
            self.scheduler_hints = outs['schedulerHints']
        if 'securityGroups' in outs:
            self.security_groups = outs['securityGroups']
        if 'stopBeforeDestroy' in outs:
            self.stop_before_destroy = outs['stopBeforeDestroy']
        if 'userData' in outs:
            self.user_data = outs['userData']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class SecGroup(pulumi.CustomResource):
    """
    Manages a V2 security group resource within OpenStack.
    
    Please note that managing security groups through the OpenStack Compute API
    has been deprecated. Unless you are using an older OpenStack environment, it is
    recommended to use the [`openstack_networking_secgroup_v2`](networking_secgroup_v2.html)
    and [`openstack_networking_secgroup_rule_v2`](networking_secgroup_rule_v2.html)
    resources instead, which uses the OpenStack Networking API.
    """
    def __init__(__self__, __name__, __opts__=None, description=None, name=None, region=None, rules=None):
        """Create a SecGroup resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if not description:
            raise TypeError('Missing required property description')
        elif not isinstance(description, basestring):
            raise TypeError('Expected property description to be a basestring')
        __self__.description = description
        """
        A description for the security group. Changing this
        updates the `description` of an existing security group.
        """
        __props__['description'] = description

        if name and not isinstance(name, basestring):
            raise TypeError('Expected property name to be a basestring')
        __self__.name = name
        """
        A unique name for the security group. Changing this
        updates the `name` of an existing security group.
        """
        __props__['name'] = name

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to obtain the V2 Compute client.
        A Compute client is needed to create a security group. If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        security group.
        """
        __props__['region'] = region

        if rules and not isinstance(rules, list):
            raise TypeError('Expected property rules to be a list')
        __self__.rules = rules
        """
        A rule describing how the security group operates. The
        rule object structure is documented below. Changing this updates the
        security group rules. As shown in the example above, multiple rule blocks
        may be used.
        """
        __props__['rules'] = rules

        super(SecGroup, __self__).__init__(
            'openstack:compute/secGroup:SecGroup',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'description' in outs:
            self.description = outs['description']
        if 'name' in outs:
            self.name = outs['name']
        if 'region' in outs:
            self.region = outs['region']
        if 'rules' in outs:
            self.rules = outs['rules']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class PoolV1(pulumi.CustomResource):
    """
    Manages a V1 load balancer pool resource within OpenStack.
    """
    def __init__(__self__, __name__, __opts__=None, lb_method=None, lb_provider=None, monitor_ids=None, name=None, protocol=None, region=None, subnet_id=None, tenant_id=None):
        """Create a PoolV1 resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if not lb_method:
            raise TypeError('Missing required property lb_method')
        elif not isinstance(lb_method, basestring):
            raise TypeError('Expected property lb_method to be a basestring')
        __self__.lb_method = lb_method
        """
        The algorithm used to distribute load between the
        members of the pool. The current specification supports 'ROUND_ROBIN' and
        'LEAST_CONNECTIONS' as valid values for this attribute.
        """
        __props__['lbMethod'] = lb_method

        if lb_provider and not isinstance(lb_provider, basestring):
            raise TypeError('Expected property lb_provider to be a basestring')
        __self__.lb_provider = lb_provider
        """
        The backend load balancing provider. For example:
        `haproxy`, `F5`, etc.
        """
        __props__['lbProvider'] = lb_provider

        if monitor_ids and not isinstance(monitor_ids, list):
            raise TypeError('Expected property monitor_ids to be a list')
        __self__.monitor_ids = monitor_ids
        """
        A list of IDs of monitors to associate with the
        pool.
        """
        __props__['monitorIds'] = monitor_ids

        if name and not isinstance(name, basestring):
            raise TypeError('Expected property name to be a basestring')
        __self__.name = name
        """
        The name of the pool. Changing this updates the name of
        the existing pool.
        """
        __props__['name'] = name

        if not protocol:
            raise TypeError('Missing required property protocol')
        elif not isinstance(protocol, basestring):
            raise TypeError('Expected property protocol to be a basestring')
        __self__.protocol = protocol
        """
        The protocol used by the pool members, you can use
        either 'TCP, 'HTTP', or 'HTTPS'. Changing this creates a new pool.
        """
        __props__['protocol'] = protocol

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to obtain the V2 Networking client.
        A Networking client is needed to create an LB pool. If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        LB pool.
        """
        __props__['region'] = region

        if not subnet_id:
            raise TypeError('Missing required property subnet_id')
        elif not isinstance(subnet_id, basestring):
            raise TypeError('Expected property subnet_id to be a basestring')
        __self__.subnet_id = subnet_id
        """
        The network on which the members of the pool will be
        located. Only members that are on this network can be added to the pool.
        Changing this creates a new pool.
        """
        __props__['subnetId'] = subnet_id

        if tenant_id and not isinstance(tenant_id, basestring):
            raise TypeError('Expected property tenant_id to be a basestring')
        __self__.tenant_id = tenant_id
        """
        The owner of the member. Required if admin wants to
        create a pool member for another tenant. Changing this creates a new member.
        """
        __props__['tenantId'] = tenant_id

        super(PoolV1, __self__).__init__(
            'openstack:loadbalancer/poolV1:PoolV1',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'lbMethod' in outs:
            self.lb_method = outs['lbMethod']
        if 'lbProvider' in outs:
            self.lb_provider = outs['lbProvider']
        if 'monitorIds' in outs:
            self.monitor_ids = outs['monitorIds']
        if 'name' in outs:
            self.name = outs['name']
        if 'protocol' in outs:
            self.protocol = outs['protocol']
        if 'region' in outs:
            self.region = outs['region']
        if 'subnetId' in outs:
            self.subnet_id = outs['subnetId']
        if 'tenantId' in outs:
            self.tenant_id = outs['tenantId']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class Port(pulumi.CustomResource):
    """
    Manages a V2 port resource within OpenStack.
    """
    def __init__(__self__, __name__, __opts__=None, admin_state_up=None, allowed_address_pairs=None, device_id=None, device_owner=None, fixed_ips=None, mac_address=None, name=None, network_id=None, no_security_groups=None, region=None, security_group_ids=None, tenant_id=None, value_specs=None):
        """Create a Port resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if admin_state_up and not isinstance(admin_state_up, bool):
            raise TypeError('Expected property admin_state_up to be a bool')
        __self__.admin_state_up = admin_state_up
        """
        Administrative up/down status for the port
        (must be "true" or "false" if provided). Changing this updates the
        `admin_state_up` of an existing port.
        """
        __props__['adminStateUp'] = admin_state_up

        if allowed_address_pairs and not isinstance(allowed_address_pairs, list):
            raise TypeError('Expected property allowed_address_pairs to be a list')
        __self__.allowed_address_pairs = allowed_address_pairs
        """
        An IP/MAC Address pair of additional IP
        addresses that can be active on this port. The structure is described
        below.
        """
        __props__['allowedAddressPairs'] = allowed_address_pairs

        if device_id and not isinstance(device_id, basestring):
            raise TypeError('Expected property device_id to be a basestring')
        __self__.device_id = device_id
        """
        The ID of the device attached to the port. Changing this
        creates a new port.
        """
        __props__['deviceId'] = device_id

        if device_owner and not isinstance(device_owner, basestring):
            raise TypeError('Expected property device_owner to be a basestring')
        __self__.device_owner = device_owner
        """
        The device owner of the Port. Changing this creates
        a new port.
        """
        __props__['deviceOwner'] = device_owner

        if fixed_ips and not isinstance(fixed_ips, list):
            raise TypeError('Expected property fixed_ips to be a list')
        __self__.fixed_ips = fixed_ips
        """
        An array of desired IPs for this port. The structure is
        described below.
        """
        __props__['fixedIps'] = fixed_ips

        if mac_address and not isinstance(mac_address, basestring):
            raise TypeError('Expected property mac_address to be a basestring')
        __self__.mac_address = mac_address
        """
        The additional MAC address.
        """
        __props__['macAddress'] = mac_address

        if name and not isinstance(name, basestring):
            raise TypeError('Expected property name to be a basestring')
        __self__.name = name
        """
        A unique name for the port. Changing this
        updates the `name` of an existing port.
        """
        __props__['name'] = name

        if not network_id:
            raise TypeError('Missing required property network_id')
        elif not isinstance(network_id, basestring):
            raise TypeError('Expected property network_id to be a basestring')
        __self__.network_id = network_id
        """
        The ID of the network to attach the port to. Changing
        this creates a new port.
        """
        __props__['networkId'] = network_id

        if no_security_groups and not isinstance(no_security_groups, bool):
            raise TypeError('Expected property no_security_groups to be a bool')
        __self__.no_security_groups = no_security_groups
        """
        If set to
        `true`, then no security groups are applied to the port. If set to `false` and
        no `security_group_ids` are specified, then the Port will yield to the default
        behavior of the Networking service, which is to usually apply the "default"
        security group.
        """
        __props__['noSecurityGroups'] = no_security_groups

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to obtain the V2 networking client.
        A networking client is needed to create a port. If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        port.
        """
        __props__['region'] = region

        if security_group_ids and not isinstance(security_group_ids, list):
            raise TypeError('Expected property security_group_ids to be a list')
        __self__.security_group_ids = security_group_ids
        """
        A list
        of security group IDs to apply to the port. The security groups must be
        specified by ID and not name (as opposed to how they are configured with
        the Compute Instance).
        """
        __props__['securityGroupIds'] = security_group_ids

        if tenant_id and not isinstance(tenant_id, basestring):
            raise TypeError('Expected property tenant_id to be a basestring')
        __self__.tenant_id = tenant_id
        """
        The owner of the Port. Required if admin wants
        to create a port for another tenant. Changing this creates a new port.
        """
        __props__['tenantId'] = tenant_id

        if value_specs and not isinstance(value_specs, dict):
            raise TypeError('Expected property value_specs to be a dict')
        __self__.value_specs = value_specs
        """
        Map of additional options.
        """
        __props__['valueSpecs'] = value_specs

        __self__.all_fixed_ips = pulumi.runtime.UNKNOWN
        """
        The collection of Fixed IP addresses on the port in the
        order returned by the Network v2 API.
        """
        __self__.all_security_group_ids = pulumi.runtime.UNKNOWN
        """
        The collection of Security Group IDs on the port
        which have been explicitly and implicitly added.
        """

        super(Port, __self__).__init__(
            'openstack:networking/port:Port',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'adminStateUp' in outs:
            self.admin_state_up = outs['adminStateUp']
        if 'allFixedIps' in outs:
            self.all_fixed_ips = outs['allFixedIps']
        if 'allSecurityGroupIds' in outs:
            self.all_security_group_ids = outs['allSecurityGroupIds']
        if 'allowedAddressPairs' in outs:
            self.allowed_address_pairs = outs['allowedAddressPairs']
        if 'deviceId' in outs:
            self.device_id = outs['deviceId']
        if 'deviceOwner' in outs:
            self.device_owner = outs['deviceOwner']
        if 'fixedIps' in outs:
            self.fixed_ips = outs['fixedIps']
        if 'macAddress' in outs:
            self.mac_address = outs['macAddress']
        if 'name' in outs:
            self.name = outs['name']
        if 'networkId' in outs:
            self.network_id = outs['networkId']
        if 'noSecurityGroups' in outs:
            self.no_security_groups = outs['noSecurityGroups']
        if 'region' in outs:
            self.region = outs['region']
        if 'securityGroupIds' in outs:
            self.security_group_ids = outs['securityGroupIds']
        if 'tenantId' in outs:
            self.tenant_id = outs['tenantId']
        if 'valueSpecs' in outs:
            self.value_specs = outs['valueSpecs']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class Router(pulumi.CustomResource):
    """
    Manages a V2 router resource within OpenStack.
    """
    def __init__(__self__, __name__, __opts__=None, admin_state_up=None, availability_zone_hints=None, distributed=None, enable_snat=None, external_fixed_ips=None, external_gateway=None, external_network_id=None, name=None, region=None, tenant_id=None, value_specs=None, vendor_options=None):
        """Create a Router resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if admin_state_up and not isinstance(admin_state_up, bool):
            raise TypeError('Expected property admin_state_up to be a bool')
        __self__.admin_state_up = admin_state_up
        """
        Administrative up/down status for the router
        (must be "true" or "false" if provided). Changing this updates the
        `admin_state_up` of an existing router.
        """
        __props__['adminStateUp'] = admin_state_up

        if availability_zone_hints and not isinstance(availability_zone_hints, list):
            raise TypeError('Expected property availability_zone_hints to be a list')
        __self__.availability_zone_hints = availability_zone_hints
        """
        An availability zone is used to make 
        network resources highly available. Used for resources with high availability so that they are scheduled on different availability zones. Changing
        this creates a new router.
        """
        __props__['availabilityZoneHints'] = availability_zone_hints

        if distributed and not isinstance(distributed, bool):
            raise TypeError('Expected property distributed to be a bool')
        __self__.distributed = distributed
        """
        Indicates whether or not to create a
        distributed router. The default policy setting in Neutron restricts
        usage of this property to administrative users only.
        """
        __props__['distributed'] = distributed

        if enable_snat and not isinstance(enable_snat, bool):
            raise TypeError('Expected property enable_snat to be a bool')
        __self__.enable_snat = enable_snat
        """
        Enable Source NAT for the router. Valid values are
        "true" or "false". An `external_network_id` has to be set in order to
        set this property. Changing this updates the `enable_snat` of the router.
        """
        __props__['enableSnat'] = enable_snat

        if external_fixed_ips and not isinstance(external_fixed_ips, list):
            raise TypeError('Expected property external_fixed_ips to be a list')
        __self__.external_fixed_ips = external_fixed_ips
        """
        An external fixed IP for the router. This
        can be repeated. The structure is described below. An `external_network_id`
        has to be set in order to set this property. Changing this updates the
        external fixed IPs of the router.
        """
        __props__['externalFixedIps'] = external_fixed_ips

        if external_gateway and not isinstance(external_gateway, basestring):
            raise TypeError('Expected property external_gateway to be a basestring')
        __self__.external_gateway = external_gateway
        """
        The
        network UUID of an external gateway for the router. A router with an
        external gateway is required if any compute instances or load balancers
        will be using floating IPs. Changing this updates the external gateway
        of an existing router.
        """
        __props__['externalGateway'] = external_gateway

        if external_network_id and not isinstance(external_network_id, basestring):
            raise TypeError('Expected property external_network_id to be a basestring')
        __self__.external_network_id = external_network_id
        """
        The network UUID of an external gateway
        for the router. A router with an external gateway is required if any
        compute instances or load balancers will be using floating IPs. Changing
        this updates the external gateway of the router.
        """
        __props__['externalNetworkId'] = external_network_id

        if name and not isinstance(name, basestring):
            raise TypeError('Expected property name to be a basestring')
        __self__.name = name
        """
        A unique name for the router. Changing this
        updates the `name` of an existing router.
        """
        __props__['name'] = name

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to obtain the V2 networking client.
        A networking client is needed to create a router. If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        router.
        """
        __props__['region'] = region

        if tenant_id and not isinstance(tenant_id, basestring):
            raise TypeError('Expected property tenant_id to be a basestring')
        __self__.tenant_id = tenant_id
        """
        The owner of the floating IP. Required if admin wants
        to create a router for another tenant. Changing this creates a new router.
        """
        __props__['tenantId'] = tenant_id

        if value_specs and not isinstance(value_specs, dict):
            raise TypeError('Expected property value_specs to be a dict')
        __self__.value_specs = value_specs
        """
        Map of additional driver-specific options.
        """
        __props__['valueSpecs'] = value_specs

        if vendor_options and not isinstance(vendor_options, dict):
            raise TypeError('Expected property vendor_options to be a dict')
        __self__.vendor_options = vendor_options
        """
        Map of additional vendor-specific options.
        Supported options are described below.
        """
        __props__['vendorOptions'] = vendor_options

        super(Router, __self__).__init__(
            'openstack:networking/router:Router',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'adminStateUp' in outs:
            self.admin_state_up = outs['adminStateUp']
        if 'availabilityZoneHints' in outs:
            self.availability_zone_hints = outs['availabilityZoneHints']
        if 'distributed' in outs:
            self.distributed = outs['distributed']
        if 'enableSnat' in outs:
            self.enable_snat = outs['enableSnat']
        if 'externalFixedIps' in outs:
            self.external_fixed_ips = outs['externalFixedIps']
        if 'externalGateway' in outs:
            self.external_gateway = outs['externalGateway']
        if 'externalNetworkId' in outs:
            self.external_network_id = outs['externalNetworkId']
        if 'name' in outs:
            self.name = outs['name']
        if 'region' in outs:
            self.region = outs['region']
        if 'tenantId' in outs:
            self.tenant_id = outs['tenantId']
        if 'valueSpecs' in outs:
            self.value_specs = outs['valueSpecs']
        if 'vendorOptions' in outs:
            self.vendor_options = outs['vendorOptions']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class LoadBalancer(pulumi.CustomResource):
    """
    Manages a V2 loadbalancer resource within OpenStack.
    """
    def __init__(__self__, __name__, __opts__=None, admin_state_up=None, description=None, flavor=None, loadbalancer_provider=None, name=None, region=None, security_group_ids=None, tenant_id=None, vip_address=None, vip_subnet_id=None):
        """Create a LoadBalancer resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if admin_state_up and not isinstance(admin_state_up, bool):
            raise TypeError('Expected property admin_state_up to be a bool')
        __self__.admin_state_up = admin_state_up
        """
        The administrative state of the Loadbalancer.
        A valid value is true (UP) or false (DOWN).
        """
        __props__['adminStateUp'] = admin_state_up

        if description and not isinstance(description, basestring):
            raise TypeError('Expected property description to be a basestring')
        __self__.description = description
        """
        Human-readable description for the Loadbalancer.
        """
        __props__['description'] = description

        if flavor and not isinstance(flavor, basestring):
            raise TypeError('Expected property flavor to be a basestring')
        __self__.flavor = flavor
        """
        The UUID of a flavor. Changing this creates a new
        loadbalancer.
        """
        __props__['flavor'] = flavor

        if loadbalancer_provider and not isinstance(loadbalancer_provider, basestring):
            raise TypeError('Expected property loadbalancer_provider to be a basestring')
        __self__.loadbalancer_provider = loadbalancer_provider
        """
        The name of the provider. Changing this
        creates a new loadbalancer.
        """
        __props__['loadbalancerProvider'] = loadbalancer_provider

        if name and not isinstance(name, basestring):
            raise TypeError('Expected property name to be a basestring')
        __self__.name = name
        """
        Human-readable name for the Loadbalancer. Does not have
        to be unique.
        """
        __props__['name'] = name

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to obtain the V2 Networking client.
        A Networking client is needed to create an LB member. If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        LB member.
        """
        __props__['region'] = region

        if security_group_ids and not isinstance(security_group_ids, list):
            raise TypeError('Expected property security_group_ids to be a list')
        __self__.security_group_ids = security_group_ids
        """
        A list of security group IDs to apply to the
        loadbalancer. The security groups must be specified by ID and not name (as
        opposed to how they are configured with the Compute Instance).
        """
        __props__['securityGroupIds'] = security_group_ids

        if tenant_id and not isinstance(tenant_id, basestring):
            raise TypeError('Expected property tenant_id to be a basestring')
        __self__.tenant_id = tenant_id
        """
        Required for admins. The UUID of the tenant who owns
        the Loadbalancer.  Only administrative users can specify a tenant UUID
        other than their own.  Changing this creates a new loadbalancer.
        """
        __props__['tenantId'] = tenant_id

        if vip_address and not isinstance(vip_address, basestring):
            raise TypeError('Expected property vip_address to be a basestring')
        __self__.vip_address = vip_address
        """
        The ip address of the load balancer.
        Changing this creates a new loadbalancer.
        """
        __props__['vipAddress'] = vip_address

        if not vip_subnet_id:
            raise TypeError('Missing required property vip_subnet_id')
        elif not isinstance(vip_subnet_id, basestring):
            raise TypeError('Expected property vip_subnet_id to be a basestring')
        __self__.vip_subnet_id = vip_subnet_id
        """
        The network on which to allocate the
        Loadbalancer's address. A tenant can only create Loadbalancers on networks
        authorized by policy (e.g. networks that belong to them or networks that
        are shared).  Changing this creates a new loadbalancer.
        """
        __props__['vipSubnetId'] = vip_subnet_id

        __self__.vip_port_id = pulumi.runtime.UNKNOWN
        """
        The Port ID of the Load Balancer IP.
        """

        super(LoadBalancer, __self__).__init__(
            'openstack:loadbalancer/loadBalancer:LoadBalancer',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'adminStateUp' in outs:
            self.admin_state_up = outs['adminStateUp']
        if 'description' in outs:
            self.description = outs['description']
        if 'flavor' in outs:
            self.flavor = outs['flavor']
        if 'loadbalancerProvider' in outs:
            self.loadbalancer_provider = outs['loadbalancerProvider']
        if 'name' in outs:
            self.name = outs['name']
        if 'region' in outs:
            self.region = outs['region']
        if 'securityGroupIds' in outs:
            self.security_group_ids = outs['securityGroupIds']
        if 'tenantId' in outs:
            self.tenant_id = outs['tenantId']
        if 'vipAddress' in outs:
            self.vip_address = outs['vipAddress']
        if 'vipPortId' in outs:
            self.vip_port_id = outs['vipPortId']
        if 'vipSubnetId' in outs:
            self.vip_subnet_id = outs['vipSubnetId']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class Firewall(pulumi.CustomResource):
    """
    Manages a v1 firewall resource within OpenStack.
    """
    def __init__(__self__, __name__, __opts__=None, admin_state_up=None, associated_routers=None, description=None, name=None, no_routers=None, policy_id=None, region=None, tenant_id=None, value_specs=None):
        """Create a Firewall resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if admin_state_up and not isinstance(admin_state_up, bool):
            raise TypeError('Expected property admin_state_up to be a bool')
        __self__.admin_state_up = admin_state_up
        """
        Administrative up/down status for the firewall
        (must be "true" or "false" if provided - defaults to "true").
        Changing this updates the `admin_state_up` of an existing firewall.
        """
        __props__['adminStateUp'] = admin_state_up

        if associated_routers and not isinstance(associated_routers, list):
            raise TypeError('Expected property associated_routers to be a list')
        __self__.associated_routers = associated_routers
        """
        Router(s) to associate this firewall instance
        with. Must be a list of strings. Changing this updates the associated routers
        of an existing firewall. Conflicts with `no_routers`.
        """
        __props__['associatedRouters'] = associated_routers

        if description and not isinstance(description, basestring):
            raise TypeError('Expected property description to be a basestring')
        __self__.description = description
        """
        A description for the firewall. Changing this
        updates the `description` of an existing firewall.
        """
        __props__['description'] = description

        if name and not isinstance(name, basestring):
            raise TypeError('Expected property name to be a basestring')
        __self__.name = name
        """
        A name for the firewall. Changing this
        updates the `name` of an existing firewall.
        """
        __props__['name'] = name

        if no_routers and not isinstance(no_routers, bool):
            raise TypeError('Expected property no_routers to be a bool')
        __self__.no_routers = no_routers
        """
        Should this firewall not be associated with any routers
        (must be "true" or "false" if provide - defaults to "false").
        Conflicts with `associated_routers`.
        """
        __props__['noRouters'] = no_routers

        if not policy_id:
            raise TypeError('Missing required property policy_id')
        elif not isinstance(policy_id, basestring):
            raise TypeError('Expected property policy_id to be a basestring')
        __self__.policy_id = policy_id
        """
        The policy resource id for the firewall. Changing
        this updates the `policy_id` of an existing firewall.
        """
        __props__['policyId'] = policy_id

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to obtain the v1 networking client.
        A networking client is needed to create a firewall. If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        firewall.
        """
        __props__['region'] = region

        if tenant_id and not isinstance(tenant_id, basestring):
            raise TypeError('Expected property tenant_id to be a basestring')
        __self__.tenant_id = tenant_id
        """
        The owner of the floating IP. Required if admin wants
        to create a firewall for another tenant. Changing this creates a new
        firewall.
        """
        __props__['tenantId'] = tenant_id

        if value_specs and not isinstance(value_specs, dict):
            raise TypeError('Expected property value_specs to be a dict')
        __self__.value_specs = value_specs
        """
        Map of additional options.
        """
        __props__['valueSpecs'] = value_specs

        super(Firewall, __self__).__init__(
            'openstack:firewall/firewall:Firewall',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'adminStateUp' in outs:
            self.admin_state_up = outs['adminStateUp']
        if 'associatedRouters' in outs:
            self.associated_routers = outs['associatedRouters']
        if 'description' in outs:
            self.description = outs['description']
        if 'name' in outs:
            self.name = outs['name']
        if 'noRouters' in outs:
            self.no_routers = outs['noRouters']
        if 'policyId' in outs:
            self.policy_id = outs['policyId']
        if 'region' in outs:
            self.region = outs['region']
        if 'tenantId' in outs:
            self.tenant_id = outs['tenantId']
        if 'valueSpecs' in outs:
            self.value_specs = outs['valueSpecs']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class Network(pulumi.CustomResource):
    """
    Manages a V2 Neutron network resource within OpenStack.
    """
    def __init__(__self__, __name__, __opts__=None, admin_state_up=None, availability_zone_hints=None, external=None, name=None, region=None, segments=None, shared=None, tenant_id=None, value_specs=None):
        """Create a Network resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if admin_state_up and not isinstance(admin_state_up, basestring):
            raise TypeError('Expected property admin_state_up to be a basestring')
        __self__.admin_state_up = admin_state_up
        """
        The administrative state of the network.
        Acceptable values are "true" and "false". Changing this value updates the
        state of the existing network.
        """
        __props__['adminStateUp'] = admin_state_up

        if availability_zone_hints and not isinstance(availability_zone_hints, list):
            raise TypeError('Expected property availability_zone_hints to be a list')
        __self__.availability_zone_hints = availability_zone_hints
        """
        An availability zone is used to make
        network resources highly available. Used for resources with high availability
        so that they are scheduled on different availability zones. Changing this
        creates a new network.
        """
        __props__['availabilityZoneHints'] = availability_zone_hints

        if external and not isinstance(external, bool):
            raise TypeError('Expected property external to be a bool')
        __self__.external = external
        """
        Specifies whether the network resource has the
        external routing facility. Valid values are true and false. Defaults to
        false. Changing this updates the external attribute of the existing network.
        """
        __props__['external'] = external

        if name and not isinstance(name, basestring):
            raise TypeError('Expected property name to be a basestring')
        __self__.name = name
        """
        The name of the network. Changing this updates the name of
        the existing network.
        """
        __props__['name'] = name

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to obtain the V2 Networking client.
        A Networking client is needed to create a Neutron network. If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        network.
        """
        __props__['region'] = region

        if segments and not isinstance(segments, list):
            raise TypeError('Expected property segments to be a list')
        __self__.segments = segments
        """
        An array of one or more provider segment objects.
        """
        __props__['segments'] = segments

        if shared and not isinstance(shared, basestring):
            raise TypeError('Expected property shared to be a basestring')
        __self__.shared = shared
        """
        Specifies whether the network resource can be accessed
        by any tenant or not. Changing this updates the sharing capabalities of the
        existing network.
        """
        __props__['shared'] = shared

        if tenant_id and not isinstance(tenant_id, basestring):
            raise TypeError('Expected property tenant_id to be a basestring')
        __self__.tenant_id = tenant_id
        """
        The owner of the network. Required if admin wants to
        create a network for another tenant. Changing this creates a new network.
        """
        __props__['tenantId'] = tenant_id

        if value_specs and not isinstance(value_specs, dict):
            raise TypeError('Expected property value_specs to be a dict')
        __self__.value_specs = value_specs
        """
        Map of additional options.
        """
        __props__['valueSpecs'] = value_specs

        super(Network, __self__).__init__(
            'openstack:networking/network:Network',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'adminStateUp' in outs:
            self.admin_state_up = outs['adminStateUp']
        if 'availabilityZoneHints' in outs:
            self.availability_zone_hints = outs['availabilityZoneHints']
        if 'external' in outs:
            self.external = outs['external']
        if 'name' in outs:
            self.name = outs['name']
        if 'region' in outs:
            self.region = outs['region']
        if 'segments' in outs:
            self.segments = outs['segments']
        if 'shared' in outs:
            self.shared = outs['shared']
        if 'tenantId' in outs:
            self.tenant_id = outs['tenantId']
        if 'valueSpecs' in outs:
            self.value_specs = outs['valueSpecs']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class GetProjectResult(object):
    """
    A collection of values returned by getProject.
    """
    def __init__(__self__, description=None, domain_id=None, region=None):
        if description and not isinstance(description, basestring):
            raise TypeError('Expected argument description to be a basestring')
        __self__.description = description
        """
        The description of the project.
        """
        if domain_id and not isinstance(domain_id, basestring):
            raise TypeError('Expected argument domain_id to be a basestring')
        __self__.domain_id = domain_id
        """
        See Argument Reference above.
        """
        if region and not isinstance(region, basestring):
            raise TypeError('Expected argument region to be a basestring')
        __self__.region = region
        """
        The region the project is located in.
        """

def get_project(domain_id=None, enabled=None, is_domain=None, name=None, parent_id=None, region=None):
    """
    Use this data source to get the ID of an OpenStack project.
    """
    __args__ = dict()

    __args__['domainId'] = domain_id
    __args__['enabled'] = enabled
    __args__['isDomain'] = is_domain
    __args__['name'] = name
    __args__['parentId'] = parent_id
    __args__['region'] = region
    __ret__ = pulumi.runtime.invoke('openstack:identity/getProject:getProject', __args__)

    return GetProjectResult(
        description=__ret__.get('description'),
        domain_id=__ret__.get('domainId'),
        region=__ret__.get('region'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class GetUserResult(object):
    """
    A collection of values returned by getUser.
    """
    def __init__(__self__, default_project_id=None, domain_id=None, region=None):
        if default_project_id and not isinstance(default_project_id, basestring):
            raise TypeError('Expected argument default_project_id to be a basestring')
        __self__.default_project_id = default_project_id
        """
        See Argument Reference above.
        """
        if domain_id and not isinstance(domain_id, basestring):
            raise TypeError('Expected argument domain_id to be a basestring')
        __self__.domain_id = domain_id
        """
        See Argument Reference above.
        """
        if region and not isinstance(region, basestring):
            raise TypeError('Expected argument region to be a basestring')
        __self__.region = region
        """
        The region the user is located in.
        """

def get_user(domain_id=None, enabled=None, idp_id=None, name=None, password_expires_at=None, protocol_id=None, region=None, unique_id=None):
    """
    Use this data source to get the ID of an OpenStack user.
    """
    __args__ = dict()

    __args__['domainId'] = domain_id
    __args__['enabled'] = enabled
    __args__['idpId'] = idp_id
    __args__['name'] = name
    __args__['passwordExpiresAt'] = password_expires_at
    __args__['protocolId'] = protocol_id
    __args__['region'] = region
    __args__['uniqueId'] = unique_id
    __ret__ = pulumi.runtime.invoke('openstack:identity/getUser:getUser', __args__)

    return GetUserResult(
        default_project_id=__ret__.get('defaultProjectId'),
        domain_id=__ret__.get('domainId'),
        region=__ret__.get('region'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class EndpointGroup(pulumi.CustomResource):
    """
    Manages a V2 Neutron Endpoint Group resource within OpenStack.
    """
    def __init__(__self__, __name__, __opts__=None, description=None, endpoints=None, name=None, region=None, tenant_id=None, type=None, value_specs=None):
        """Create a EndpointGroup resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, basestring):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if description and not isinstance(description, basestring):
            raise TypeError('Expected property description to be a basestring')
        __self__.description = description
        """
        The human-readable description for the group.
        Changing this updates the description of the existing group.
        """
        __props__['description'] = description

        if endpoints and not isinstance(endpoints, list):
            raise TypeError('Expected property endpoints to be a list')
        __self__.endpoints = endpoints
        """
        List of endpoints of the same type, for the endpoint group. The values will depend on the type.
        Changing this creates a new group.
        """
        __props__['endpoints'] = endpoints

        if name and not isinstance(name, basestring):
            raise TypeError('Expected property name to be a basestring')
        __self__.name = name
        """
        The name of the group. Changing this updates the name of
        the existing group.
        """
        __props__['name'] = name

        if region and not isinstance(region, basestring):
            raise TypeError('Expected property region to be a basestring')
        __self__.region = region
        """
        The region in which to obtain the V2 Networking client.
        A Networking client is needed to create an endpoint group. If omitted, the
        `region` argument of the provider is used. Changing this creates a new
        group.
        """
        __props__['region'] = region

        if tenant_id and not isinstance(tenant_id, basestring):
            raise TypeError('Expected property tenant_id to be a basestring')
        __self__.tenant_id = tenant_id
        """
        The owner of the group. Required if admin wants to
        create an endpoint group for another project. Changing this creates a new group.
        """
        __props__['tenantId'] = tenant_id

        if type and not isinstance(type, basestring):
            raise TypeError('Expected property type to be a basestring')
        __self__.type = type
        """
        The type of the endpoints in the group. A valid value is subnet, cidr, network, router, or vlan.
        Changing this creates a new group.
        """
        __props__['type'] = type

        if value_specs and not isinstance(value_specs, dict):
            raise TypeError('Expected property value_specs to be a dict')
        __self__.value_specs = value_specs
        """
        Map of additional options.
        """
        __props__['valueSpecs'] = value_specs

        super(EndpointGroup, __self__).__init__(
            'openstack:vpnaas/endpointGroup:EndpointGroup',
            __name__,
            __props__,
            __opts__)

    def set_outputs(self, outs):
        if 'description' in outs:
            self.description = outs['description']
        if 'endpoints' in outs:
            self.endpoints = outs['endpoints']
        if 'name' in outs:
            self.name = outs['name']
        if 'region' in outs:
            self.region = outs['region']
        if 'tenantId' in outs:
            self.tenant_id = outs['tenantId']
        if 'type' in outs:
            self.type = outs['type']
        if 'valueSpecs' in outs:
            self.value_specs = outs['valueSpecs']



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime

class GetDnsZoneResult(object):
    """
    A collection of values returned by getDnsZone.
    """
    def __init__(__self__, attributes=None, created_at=None, masters=None, pool_id=None, project_id=None, region=None, serial=None, transferred_at=None, updated_at=None, version=None):
        if attributes and not isinstance(attributes, dict):
            raise TypeError('Expected argument attributes to be a dict')
        __self__.attributes = attributes
        """
        Attributes of the DNS Service scheduler.
        """
        if created_at and not isinstance(created_at, basestring):
            raise TypeError('Expected argument created_at to be a basestring')
        __self__.created_at = created_at
        """
        The time the zone was created.
        """
        if masters and not isinstance(masters, list):
            raise TypeError('Expected argument masters to be a list')
        __self__.masters = masters
        """
        An array of master DNS servers. When `type` is  `SECONDARY`.
        """
        if pool_id and not isinstance(pool_id, basestring):
            raise TypeError('Expected argument pool_id to be a basestring')
        __self__.pool_id = pool_id
        """
        The ID of the pool hosting the zone.
        """
        if project_id and not isinstance(project_id, basestring):
            raise TypeError('Expected argument project_id to be a basestring')
        __self__.project_id = project_id
        """
        The project ID that owns the zone.
        """
        if region and not isinstance(region, basestring):
            raise TypeError('Expected argument region to be a basestring')
        __self__.region = region
        """
        See Argument Reference above.
        """
        if serial and not isinstance(serial, int):
            raise TypeError('Expected argument serial to be a int')
        __self__.serial = serial
        """
        The serial number of the zone.
        """
        if transferred_at and not isinstance(transferred_at, basestring):
            raise TypeError('Expected argument transferred_at to be a basestring')
        __self__.transferred_at = transferred_at
        """
        The time the zone was transferred.
        """
        if updated_at and not isinstance(updated_at, basestring):
            raise TypeError('Expected argument updated_at to be a basestring')
        __self__.updated_at = updated_at
        """
        The time the zone was last updated.
        """
        if version and not isinstance(version, int):
            raise TypeError('Expected argument version to be a int')
        __self__.version = version
        """
        The version of the zone.
        """

def get_dns_zone(attributes=None, created_at=None, description=None, email=None, masters=None, name=None, pool_id=None, project_id=None, region=None, serial=None, status=None, transferred_at=None, ttl=None, type=None, updated_at=None, version=None):
    """
    Use this data source to get the ID of an available OpenStack DNS zone.
    """
    __args__ = dict()

    __args__['attributes'] = attributes
    __args__['createdAt'] = created_at
    __args__['description'] = description
    __args__['email'] = email
    __args__['masters'] = masters
    __args__['name'] = name
    __args__['poolId'] = pool_id
    __args__['projectId'] = project_id
    __args__['region'] = region
    __args__['serial'] = serial
    __args__['status'] = status
    __args__['transferredAt'] = transferred_at
    __args__['ttl'] = ttl
    __args__['type'] = type
    __args__['updatedAt'] = updated_at
    __args__['version'] = version
    __ret__ = pulumi.runtime.invoke('openstack:dns/getDnsZone:getDnsZone', __args__)

    return GetDnsZoneResult(
        attributes=__ret__.get('attributes'),
        created_at=__ret__.get('createdAt'),
        masters=__ret__.get('masters'),
        pool_id=__ret__.get('poolId'),
        project_id=__ret__.get('projectId'),
        region=__ret__.get('region'),
        serial=__ret__.get('serial'),
        transferred_at=__ret__.get('transferredAt'),
        updated_at=__ret__.get('updatedAt'),
        version=__ret__.get('version'))
